'''
交易
https://www.okx.com/docs-v5/zh/#order-book-trading-trade
'''

from paux.param import to_local
from okx.api._client import Client


class _TradeEndpoints():
    set_order = ['/api/v5/trade/order', 'POST']  # POST / 下单
    set_batch_orders = ['/api/v5/trade/batch-orders', 'POST']  # POST / 批量下单
    set_cancel_order = ['/api/v5/trade/cancel-order', 'POST']  # POST / 撤单
    set_cancel_batch_orders = ['/api/v5/trade/cancel-batch-orders', 'POST']  # POST / 批量撤单
    set_amend_order = ['/api/v5/trade/amend-order', 'POST']  # POST / 修改订单
    set_amend_batch_orders = ['/api/v5/trade/amend-batch-orders', 'POST']  # POST / 批量修改订单
    set_close_position = ['/api/v5/trade/close-position', 'POST']  # POST / 市价仓位全平
    get_order = ['/api/v5/trade/order', 'GET']  # GET / 获取订单信息
    get_orders_pending = ['/api/v5/trade/orders-pending', 'GET']  # GET / 获取未成交订单列表
    get_orders_history = ['/api/v5/trade/orders-history', 'GET']  # GET / 获取历史订单记录（近七天）
    get_orders_history_archive = ['/api/v5/trade/orders-history-archive', 'GET']  # GET / 获取历史订单记录（近三个月）
    get_fills = ['/api/v5/trade/fills', 'GET']  # GET / 获取成交明细（近三天）
    get_fills_history = ['/api/v5/trade/fills-history', 'GET']  # GET / 获取成交明细（近三个月）
    set_fills_archive = ['/api/v5/trade/fills-archive', 'POST']  # POST / 申请成交明细（近两年）
    get_fills_archive = ['/api/v5/trade/fills-archive', 'GET']  # GET / 获取成交明细（近两年）
    get_easy_convert_currency_list = ['/api/v5/trade/easy-convert-currency-list', 'GET']  # GET / 获取一键兑换主流币币种列表
    set_easy_convert = ['/api/v5/trade/easy-convert', 'POST']  # POST / 一键兑换主流币交易
    get_easy_convert_history = ['/api/v5/trade/easy-convert-history', 'GET']  # GET / 获取一键兑换主流币历史记录
    get_one_click_repay_currency_list = ['/api/v5/trade/one-click-repay-currency-list', 'GET']  # GET / 获取一键还债币种列表
    set_one_click_repay = ['/api/v5/trade/one-click-repay', 'POST']  # POST / 一键还债交易
    get_one_click_repay_history = ['/api/v5/trade/one-click-repay-history', 'GET']  # GET / 获取一键还债历史记录
    set_mass_cancel = ['/api/v5/trade/mass-cancel', 'POST']  # POST / 撤销 MMP 订单
    set_cancel_all_after = ['/api/v5/trade/cancel-all-after', 'POST']  # POST / 倒计时全部撤单


class Trade(Client):

    # POST / 下单
    def set_order(self, instId: str, tdMode: str, side: str, ordType: str, sz: str, ccy: str = '', clOrdId: str = '',
                  tag: str = '', posSide: str = '', px: str = '', pxUsd: str = '', pxVol: str = '',
                  reduceOnly: bool = '', tgtCcy: str = '', banAmend: bool = '', tpTriggerPx: str = '',
                  tpOrdPx: str = '', attachAlgoClOrdId: str = '', slTriggerPx: str = '', slOrdPx: str = '',
                  tpTriggerPxType: str = '', slTriggerPxType: str = '', quickMgnType: str = '', stpId: str = '',
                  stpMode: str = '', proxies={}, proxy_host: str = None):
        '''
        只有当您的账户有足够的资金才能下单。<!- 1-2-3 -->该接口支持带单合约的下单，但不支持为带单合约平仓。请参考跟单了解更多
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-post-place-order
        
        限速：60次/2s跟单交易带单合约的限速：4次/2s
        
    
        请求参数:
        Parameter         	Type    	Required	Description

        instId            	String  	是       	产品ID，如BTC-USD-190927-5000-C
        tdMode            	String  	是       	交易模式保证金模式：isolated：逐仓 ；cross：全仓非保证金模式：cash：非保证金
        ccy               	String  	否       	保证金币种，仅适用于单币种保证金模式下的全仓杠杆订单
        clOrdId           	String  	否       	客户自定义订单ID字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。
        tag               	String  	否       	订单标签字母（区分大小写）与数字的组合，可以是纯字母、纯数字，且长度在1-16位之间。
        side              	String  	是       	订单方向buy：买，sell：卖
        posSide           	String  	可选      	持仓方向在开平仓模式下必填，且仅可选择long或short。 仅适用交割、永续。
        ordType           	String  	是       	订单类型market：市价单limit：限价单post_only：只做maker单fok：全部成交或立即取消ioc：立即成交并取消剩余optimal_limit_ioc：市价委托立即成交并取消剩余（仅适用交割、永续）mmp：做市商保护(仅适用于组合保证金账户模式下的期权订单)mmp_and_post_only：做市商保护且只做maker单(仅适用于组合保证金账户模式下的期权订单)
        sz                	String  	是       	委托数量
        px                	String  	可选      	委托价格，仅适用于limit、post_only、fok、ioc、mmp、mmp_and_post_only类型的订单期权下单时，px/pxUsd/pxVol 只能填一个
        pxUsd             	String  	可选      	以USD价格进行期权下单仅适用于期权期权下单时 px/pxUsd/pxVol 必填一个，且只能填一个
        pxVol             	String  	可选      	以隐含波动率进行期权下单，例如 1 代表 100%仅适用于期权期权下单时 px/pxUsd/pxVol 必填一个，且只能填一个
        reduceOnly        	Boolean 	否       	是否只减仓，true或false，默认false仅适用于币币杠杆，以及买卖模式下的交割/永续仅适用于单币种保证金模式和跨币种保证金模式
        tgtCcy            	String  	否       	市价单委托数量sz的单位，仅适用于币币市价订单base_ccy: 交易货币 ；quote_ccy：计价货币买单默认quote_ccy， 卖单默认base_ccy
        banAmend          	Boolean 	否       	是否禁止币币市价改单，true 或 false，默认false为true时，余额不足时，系统不会改单，下单会失败，仅适用于币币市价单
        tpTriggerPx       	String  	否       	止盈触发价，如果填写此参数，必须填写 止盈委托价
        tpOrdPx           	String  	否       	止盈委托价，如果填写此参数，必须填写 止盈触发价委托价格为-1时，执行市价止盈
        attachAlgoClOrdId 	String  	否       	下单附带止盈止损时，客户自定义的策略订单ID字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。订单完全成交，下止盈止损委托单时，该值会传给algoClOrdId
        slTriggerPx       	String  	否       	止损触发价，如果填写此参数，必须填写 止损委托价
        slOrdPx           	String  	否       	止损委托价，如果填写此参数，必须填写 止损触发价委托价格为-1时，执行市价止损
        tpTriggerPxType   	String  	否       	止盈触发价类型last：最新价格index：指数价格mark：标记价格默认为last
        slTriggerPxType   	String  	否       	止损触发价类型last：最新价格index：指数价格mark：标记价格默认为last
        quickMgnType      	String  	否       	一键借币类型，仅适用于杠杆逐仓的一键借币模式：manual：手动，auto_borrow： 自动借币，auto_repay： 自动还币默认是manual：手动
        stpId             	String  	否       	自成交保护ID。来自同一个母账户配着同一个ID的订单不能自成交用户自定义1<=x<=999999999的整数
        stpMode           	String  	否       	自成交保护模式，需要stpId有值才会生效默认为 cancel makercancel_maker,cancel_taker,cancel_bothCancel both不支持FOK
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0表示成功
        msg               	String  	错误信息，代码为0时，该字段为空
        data              	String  	包含结果的对象数组
        > ordId           	String  	订单ID
        > clOrdId         	String  	客户自定义订单ID
        > tag             	String  	订单标签
        > sCode           	String  	事件执行结果的code，0代表成功
        > sMsg            	String  	事件执行失败或成功时的msg
        inTime            	String  	REST网关接收请求时的时间戳，Unix时间戳的微秒数格式，如1597026383085123返回的时间是请求验证后的时间。
        outTime           	String  	REST网关发送响应时的时间戳，Unix时间戳的微秒数格式，如1597026383085123
        '''
        return self.send_request(*_TradeEndpoints.set_order, **to_local(locals()))

    # POST / 批量下单
    def set_batch_orders(self, orders: list, proxies={}, proxy_host: str = None, ):
        '''
        每次最多可以批量提交20个新订单。请求参数应该按数组格式传递，会依次委托订单。<!- 1-2-3 -->该接口支持带单合约的下单，但不支持为带单合约平仓。请参考跟单了解更多
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-post-place-multiple-orders
        
        限速：300个/2s跟单交易带单合约的限速：4个/2s
        
    
        请求参数:
        Parameter         	Type    	Required	Description

        instId            	String  	是       	产品ID，如BTC-USD-190927-5000-C
        tdMode            	String  	是       	交易模式保证金模式：isolated：逐仓 ；cross：全仓非保证金模式：cash：非保证金
        ccy               	String  	否       	保证金币种，仅适用于单币种保证金模式下的全仓杠杆订单
        clOrdId           	String  	否       	客户自定义订单ID字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。
        tag               	String  	否       	订单标签字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-16位之间。
        side              	String  	是       	订单方向buy：买，sell：卖
        posSide           	String  	可选      	持仓方向在开平仓模式下必填，且仅可选择long或short。 仅适用交割、永续。
        ordType           	String  	是       	订单类型market：市价单limit：限价单post_only：只做maker单fok：全部成交或立即取消ioc：立即成交并取消剩余optimal_limit_ioc：市价委托立即成交并取消剩余（仅适用交割、永续）mmp：做市商保护(仅适用于组合保证金账户模式下的期权订单)mmp_and_post_only：做市商保护且只做maker单(仅适用于组合保证金账户模式下的期权订单)
        sz                	String  	是       	委托数量
        px                	String  	可选      	委托价格，仅适用于limit、post_only、fok、ioc、mmp、mmp_and_post_only类型的订单期权下单时，px/pxUsd/pxVol 只能填一个
        pxUsd             	String  	可选      	以USD价格进行期权下单仅适用于期权期权下单时 px/pxUsd/pxVol 必填一个，且只能填一个
        pxVol             	String  	可选      	以隐含波动率进行期权下单，例如 1 代表 100%仅适用于期权期权下单时 px/pxUsd/pxVol 必填一个，且只能填一个
        reduceOnly        	Boolean 	否       	是否只减仓，true或false，默认false仅适用于币币杠杆，以及买卖模式下的交割/永续仅适用于单币种保证金模式和跨币种保证金模式
        tgtCcy            	String  	否       	市价单委托数量sz的单位，仅适用于币币市价订单base_ccy: 交易货币 ；quote_ccy：计价货币买单默认quote_ccy， 卖单默认base_ccy
        banAmend          	Boolean 	否       	是否禁止币币市价改单，true 或 false，默认false为true时，余额不足时，系统不会改单，下单会失败，仅适用于币币市价单
        attachAlgoClOrdId 	String  	否       	下单附带止盈止损时，客户自定义的策略订单ID字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。订单完全成交，下止盈止损委托单时，该值会传给algoClOrdId
        tpTriggerPx       	String  	否       	止盈触发价，如果填写此参数，必须填写 止盈委托价
        tpOrdPx           	String  	否       	止盈委托价，如果填写此参数，必须填写 止盈触发价委托价格为-1时，执行市价止盈
        slTriggerPx       	String  	否       	止损触发价，如果填写此参数，必须填写 止损委托价
        slOrdPx           	String  	否       	止损委托价，如果填写此参数，必须填写 止损触发价委托价格为-1时，执行市价止损
        stpId             	String  	否       	自成交保护ID。来自同一个母账户配着同一个ID的订单不能自成交用户自定义1<=x<=999999999的整数
        stpMode           	String  	否       	自成交保护模式，需要stpId有值才会生效默认为 cancel makercancel_maker,cancel_taker,cancel_bothCancel both不支持FOK
        tpTriggerPxType   	String  	否       	止盈触发价类型last：最新价格index：指数价格mark：标记价格默认为last
        slTriggerPxType   	String  	否       	止损触发价类型last：最新价格index：指数价格mark：标记价格默认为last
        quickMgnType      	String  	否       	一键借币类型，仅适用于杠杆逐仓的一键借币模式：manual：手动，auto_borrow： 自动借币，auto_repay： 自动还币默认是manual：手动
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0表示成功
        msg               	String  	错误信息，代码为0时，该字段为空
        data              	String  	包含结果的对象数组
        > ordId           	String  	订单ID
        > clOrdId         	String  	客户自定义订单ID
        > tag             	String  	订单标签
        > sCode           	String  	事件执行结果的code，0代表成功
        > sMsg            	String  	事件执行失败或成功时的msg
        inTime            	String  	REST网关接收请求时的时间戳，Unix时间戳的微秒数格式，如1597026383085123返回的时间是请求验证后的时间。
        outTime           	String  	REST网关发送响应时的时间戳，Unix时间戳的微秒数格式，如1597026383085123
        '''
        return self.send_request(*_TradeEndpoints.set_batch_orders, orders, proxies=proxies, proxy_host=proxy_host)

    # POST / 撤单
    def set_cancel_order(self, instId: str, ordId: str = '', clOrdId: str = '', proxies={}, proxy_host: str = None):
        '''
        撤销之前下的未完成订单。
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-post-cancel-order
        
        限速：60次/2s
        
    
        请求参数:
        Parameter         	Type    	Required	Description

        instId            	String  	是       	产品ID，如BTC-USD-190927
        ordId             	String  	可选      	订单ID，ordId和clOrdId必须传一个，若传两个，以ordId为主
        clOrdId           	String  	可选      	用户自定义ID
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0表示成功
        msg               	String  	错误信息，代码为0时，该字段为空
        data              	String  	包含结果的对象数组
        > ordId           	String  	订单ID
        > clOrdId         	String  	客户自定义订单ID
        > sCode           	String  	事件执行结果的code，0代表成功
        > sMsg            	String  	事件执行失败时的msg
        inTime            	String  	REST网关接收请求时的时间戳，Unix时间戳的微秒数格式，如1597026383085123返回的时间是请求验证后的时间。
        outTime           	String  	REST网关发送响应时的时间戳，Unix时间戳的微秒数格式，如1597026383085123
        '''
        return self.send_request(*_TradeEndpoints.set_cancel_order, **to_local(locals()))

    # POST / 批量撤单
    def set_cancel_batch_orders(self, orders: list, proxies={}, proxy_host: str = None):
        '''
        撤销未完成的订单，每次最多可以撤销20个订单。请求参数应该按数组格式传递。
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-post-cancel-multiple-orders
        
        限速：300个/2s
        
    
        请求参数:
        Parameter         	Type    	Required	Description

        instId            	String  	是       	产品ID，如BTC-USD-190927
        ordId             	String  	可选      	订单ID，ordId和clOrdId必须传一个，若传两个，以ordId为主
        clOrdId           	String  	可选      	用户自定义ID
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0表示成功
        msg               	String  	错误信息，代码为0时，该字段为空
        data              	String  	包含结果的对象数组
        > ordId           	String  	订单ID
        > clOrdId         	String  	客户自定义订单ID
        > sCode           	String  	事件执行结果的code，0代表成功
        > sMsg            	String  	事件执行失败时的msg
        inTime            	String  	REST网关接收请求时的时间戳，Unix时间戳的微秒数格式，如1597026383085123返回的时间是请求验证后的时间。
        outTime           	String  	REST网关发送响应时的时间戳，Unix时间戳的微秒数格式，如1597026383085123
        '''
        return self.send_request(*_TradeEndpoints.set_cancel_batch_orders, orders, proxies=proxies,
                                 proxy_host=proxy_host)

    # POST / 修改订单
    def set_amend_order(self, instId: str, cxlOnFail: bool = '', ordId: str = '', clOrdId: str = '', reqId: str = '',
                        newSz: str = '', newPx: str = '', newPxUsd: str = '', newPxVol: str = '',
                        newTpTriggerPx: str = '', newTpOrdPx: str = '', newSlTriggerPx: str = '', newSlOrdPx: str = '',
                        newTpTriggerPxType: str = '', newSlTriggerPxType: str = '', proxies={}, proxy_host: str = None):
        '''
        修改当前未成交的挂单
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-post-amend-order
        
        限速：60次/2s跟单交易带单合约的限速：4个/2s
        限速规则：UserID + Instrument ID
    
        请求参数:
        Parameter         	Type    	Required	Description

        instId            	String  	是       	产品ID
        cxlOnFail         	Boolean 	否       	false：不自动撤单true：自动撤单 当订单修改失败时，该订单是否需要自动撤销。默认为false
        ordId             	String  	可选      	订单ID，ordId和clOrdId必须传一个，若传两个，以ordId为主
        clOrdId           	String  	可选      	用户自定义order ID
        reqId             	String  	否       	用户自定义修改事件ID字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。
        newSz             	String  	可选      	修改的新数量，对于部分成交订单，该数量应包含已成交数量。
        newPx             	String  	可选      	修改后的新价格修改的新价格期权改单时，newPx/newPxUsd/newPxVol 只能填一个，且必须与下单参数保持一致，如下单用px，改单时需使用newPx
        newPxUsd          	String  	可选      	以USD价格进行期权改单仅适用于期权，期权改单时，newPx/newPxUsd/newPxVol 只能填一个
        newPxVol          	String  	可选      	以隐含波动率进行期权改单，例如 1 代表 100%仅适用于期权，期权改单时，newPx/newPxUsd/newPxVol 只能填一个
        newTpTriggerPx    	String  	可选      	止盈触发价如果止盈触发价或者委托价为0，那代表删除止盈。只适用于交割和永续合约。
        newTpOrdPx        	String  	可选      	止盈委托价委托价格为-1时，执行市价止盈。只适用于交割和永续合约。
        newSlTriggerPx    	String  	可选      	止损触发价如果止损触发价或者委托价为0，那代表删除止损。只适用于交割和永续合约。
        newSlOrdPx        	String  	可选      	止损委托价委托价格为-1时，执行市价止损。 只适用于交割和永续合约。
        newTpTriggerPxType	String  	可选      	止盈触发价类型last：最新价格index：指数价格mark：标记价格只适用于交割/永续如果要新增止盈，该参数必填
        newSlTriggerPxType	String  	可选      	止损触发价类型last：最新价格index：指数价格mark：标记价格只适用于交割/永续如果要新增止损，该参数必填
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0表示成功
        msg               	String  	错误信息，代码为0时，该字段为空
        data              	String  	包含结果的对象数组
        > ordId           	String  	订单ID
        > clOrdId         	String  	用户自定义ID
        > reqId           	String  	用户自定义修改事件ID
        > sCode           	String  	事件执行结果的code，0代表成功
        > sMsg            	String  	事件执行失败时的msg
        inTime            	String  	REST网关接收请求时的时间戳，Unix时间戳的微秒数格式，如1597026383085123返回的时间是请求验证后的时间。
        outTime           	String  	REST网关发送响应时的时间戳，Unix时间戳的微秒数格式，如1597026383085123
        '''
        return self.send_request(*_TradeEndpoints.set_amend_order, **to_local(locals()))

    # POST / 批量修改订单
    def set_amend_batch_orders(self, orders, proxies={}, proxy_host: str = None):
        '''
        修改未完成的订单，一次最多可批量修改20个订单。请求参数应该按数组格式传递。
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-post-amend-multiple-orders
        
        限速：300个/2s跟单交易带单合约的限速：4个/2s
        限速规则：UserID + Instrument ID
    
        请求参数:
        Parameter         	Type    	Required	Description

        instId            	String  	是       	产品ID
        cxlOnFail         	Boolean 	否       	false：不自动撤单true：自动撤单 当订单修改失败时，该订单是否需要自动撤销，默认为false
        ordId             	String  	可选      	订单ID，ordId和clOrdId必须传一个，若传两个，以ordId为主
        clOrdId           	String  	可选      	用户自定义order ID
        reqId             	String  	否       	用户自定义修改事件ID字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。
        newSz             	String  	可选      	修改的新数量，对于部分成交订单，该数量应包含已成交数量。
        newPx             	String  	可选      	修改后的新价格修改的新价格期权改单时，newPx/newPxUsd/newPxVol 只能填一个，且必须与下单参数保持一致，如下单用px，改单时需使用newPx
        newPxUsd          	String  	可选      	以USD价格进行期权改单仅适用于期权，期权改单时，newPx/newPxUsd/newPxVol 只能填一个
        newPxVol          	String  	可选      	以隐含波动率进行期权改单，例如 1 代表 100%仅适用于期权，期权改单时，newPx/newPxUsd/newPxVol 只能填一个
        newTpTriggerPx    	String  	可选      	止盈触发价如果止盈触发价或者委托价为0，那代表删除止盈。只适用于交割和永续合约。
        newTpOrdPx        	String  	可选      	止盈委托价委托价格为-1时，执行市价止盈。只适用于交割和永续合约。
        newSlTriggerPx    	String  	可选      	止损触发价如果止损触发价或者委托价为0，那代表删除止损。只适用于交割和永续合约。
        newSlOrdPx        	String  	可选      	止损委托价委托价格为-1时，执行市价止损。只适用于交割和永续合约。
        newTpTriggerPxType	String  	可选      	止盈触发价类型last：最新价格index：指数价格mark：标记价格只适用于交割/永续如果要新增止盈，该参数必填
        newSlTriggerPxType	String  	可选      	止损触发价类型last：最新价格index：指数价格mark：标记价格只适用于交割/永续如果要新增止损，该参数必填
        返回参数:
        Parameter         	Type    	Description
        code              	String  	结果代码，0表示成功
        msg               	String  	错误信息，代码为0时，该字段为空
        data              	String  	包含结果的对象数组
        > ordId           	String  	订单ID
        > clOrdId         	String  	用户自定义ID
        > reqId           	String  	用户自定义修改事件ID
        > sCode           	String  	事件执行结果的code，0代表成功
        > sMsg            	String  	事件执行失败时的msg
        inTime            	String  	REST网关接收请求时的时间戳，Unix时间戳的微秒数格式，如1597026383085123返回的时间是请求验证后的时间。
        outTime           	String  	REST网关发送响应时的时间戳，Unix时间戳的微秒数格式，如1597026383085123
        '''
        return self.send_request(*_TradeEndpoints.set_amend_batch_orders, orders, proxies=proxies,
                                 proxy_host=proxy_host)

    # POST / 市价仓位全平
    def set_close_position(self, instId: str, mgnMode: str, posSide: str = '', ccy: str = '', autoCxl: bool = '',
                           clOrdId: str = '', tag: str = '', proxies={}, proxy_host: str = None):
        '''
        市价平掉指定交易产品的持仓
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-post-close-positions
        
        限速：20次/2s
        
    
        请求参数:
        Parameter         	Type    	Required	Description

        instId            	String  	是       	产品ID
        posSide           	String  	可选      	持仓方向买卖模式下：可不填写此参数，默认值net，如果填写，仅可以填写net开平仓模式下： 必须填写此参数，且仅可以填写long：平多 ，short：平空
        mgnMode           	String  	是       	保证金模式cross：全仓 ；isolated：逐仓
        ccy               	String  	可选      	保证金币种，单币种保证金模式的全仓币币杠杆平仓必填
        autoCxl           	Boolean 	否       	当市价全平时，平仓单是否需要自动撤销,默认为false.false：不自动撤单true：自动撤单
        clOrdId           	String  	否       	客户自定义ID字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。
        tag               	String  	否       	订单标签字母（区分大小写）与数字的组合，可以是纯字母、纯数字，且长度在1-16位之间。
        返回参数:
        Parameter         	Type    	Description
        instId            	String  	产品ID
        posSide           	String  	持仓方向
        clOrdId           	String  	客户自定义ID字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。
        tag               	String  	订单标签字母（区分大小写）与数字的组合，可以是纯字母、纯数字，且长度在1-16位之间。
        '''
        return self.send_request(*_TradeEndpoints.set_close_position, **to_local(locals()))

    # GET / 获取订单信息
    def get_order(self, instId: str, ordId: str = '', clOrdId: str = '', proxies={}, proxy_host: str = None):
        '''
        查订单信息
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-get-order-details
        
        限速：60次/2s
        
    
        请求参数:
        Parameter         	Type    	Required	Description

        instId            	String  	是       	产品ID ，如BTC-USD-190927
        ordId             	String  	可选      	订单ID ，ordId和clOrdId必须传一个，若传两个，以ordId为主
        clOrdId           	String  	可选      	用户自定义ID如果clOrdId关联了多个订单，只会返回最近的那笔订单
        返回参数:
        Parameter         	Type    	Description
        instType          	String  	产品类型SPOT：币币MARGIN：币币杠杆SWAP：永续合约FUTURES：交割合约OPTION：期权
        instId            	String  	产品ID
        tgtCcy            	String  	币币市价单委托数量sz的单位base_ccy: 交易货币 ；quote_ccy：计价货币仅适用于币币市价订单默认买单为quote_ccy，卖单为base_ccy
        ccy               	String  	保证金币种，仅适用于单币种保证金模式下的全仓币币杠杆订单
        ordId             	String  	订单ID
        clOrdId           	String  	客户自定义订单ID
        tag               	String  	订单标签
        px                	String  	委托价格，对于期权，以币(如BTC, ETH)为单位
        pxUsd             	String  	期权价格，以USD为单位仅适用于期权，其他业务线返回空字符串""
        pxVol             	String  	期权订单的隐含波动率仅适用于期权，其他业务线返回空字符串""
        pxType            	String  	期权的价格类型px：代表按价格下单，单位为币 (请求参数 px 的数值单位是BTC或ETH)pxVol：代表按pxVol下单pxUsd：代表按照pxUsd下单，单位为USD (请求参数px 的数值单位是USD)
        sz                	String  	委托数量
        pnl               	String  	收益，适用于有成交的平仓订单，其他情况均为0
        ordType           	String  	订单类型market：市价单limit：限价单post_only：只做maker单fok：全部成交或立即取消ioc：立即成交并取消剩余optimal_limit_ioc：市价委托立即成交并取消剩余（仅适用交割、永续）mmp：做市商保护(仅适用于组合保证金账户模式下的期权订单)mmp_and_post_only：做市商保护且只做maker单(仅适用于组合保证金账户模式下的期权订单)
        side              	String  	订单方向
        posSide           	String  	持仓方向
        tdMode            	String  	交易模式
        accFillSz         	String  	累计成交数量对于币币和杠杆，单位为交易货币，如 BTC-USDT, 单位为 BTC；对于市价单，无论tgtCcy是base_ccy，还是quote_ccy，单位均为交易货币；对于交割、永续以及期权，单位为张。
        fillPx            	String  	最新成交价格，如果成交数量为0，该字段为""
        tradeId           	String  	最新成交ID
        fillSz            	String  	最新成交数量对于币币和杠杆，单位为交易货币，如 BTC-USDT, 单位为 BTC；对于市价单，无论tgtCcy是base_ccy，还是quote_ccy，单位均为交易货币；对于交割、永续以及期权，单位为张。
        fillTime          	String  	最新成交时间
        avgPx             	String  	成交均价，如果成交数量为0，该字段也为""
        state             	String  	订单状态canceled：撤单成功live：等待成交partially_filled：部分成交filled：完全成交mmp_canceled：做市商保护机制导致的自动撤单
        lever             	String  	杠杆倍数，0.01到125之间的数值，仅适用于币币杠杆/交割/永续
        attachAlgoClOrdId 	String  	下单附带止盈止损时，客户自定义的策略订单ID
        tpTriggerPx       	String  	止盈触发价
        tpTriggerPxType   	String  	止盈触发价类型last：最新价格index：指数价格mark：标记价格
        tpOrdPx           	String  	止盈委托价
        slTriggerPx       	String  	止损触发价
        slTriggerPxType   	String  	止损触发价类型last：最新价格index：指数价格mark：标记价格
        slOrdPx           	String  	止损委托价
        stpId             	String  	自成交保护ID如果自成交保护不适用则返回""
        stpMode           	String  	自成交保护模式如果自成交保护不适用则返回""
        feeCcy            	String  	交易手续费币种
        fee               	String  	手续费与返佣对于币币和杠杆，为订单交易累计的手续费，平台向用户收取的交易手续费，为负数。如： -0.01对于交割、永续和期权，为订单交易累计的手续费和返佣
        rebateCcy         	String  	返佣金币种
        source            	String  	订单来源6：计划委托策略触发后的生成的普通单7：止盈止损策略触发后的生成的普通单13：策略委托单触发后的生成的普通单24：移动止盈止损策略触发后的生成的普通单
        rebate            	String  	返佣金额，仅适用于币币和杠杆，平台向达到指定lv交易等级的用户支付的挂单奖励（返佣），如果没有返佣金，该字段为“”。手续费返佣为正数，如：0.01
        category          	String  	订单种类normal：普通委托twap：TWAP自动换币adl：ADL自动减仓full_liquidation：强制平仓partial_liquidation：强制减仓delivery：交割ddh：对冲减仓类型订单
        reduceOnly        	String  	是否只减仓，true或false
        cancelSource      	String  	订单取消来源的原因枚举值代码
        cancelSourceReason	String  	订单取消来源的对应具体原因
        quickMgnType      	String  	一键借币类型，仅适用于杠杆逐仓的一键借币模式manual：手动，auto_borrow： 自动借币，auto_repay： 自动还币
        algoClOrdId       	String  	客户自定义策略订单ID。策略订单触发，且策略单有algoClOrdId时有值，否则为"",
        algoId            	String  	策略委托单ID，策略订单触发时有值，否则为""
        uTime             	String  	订单状态更新时间，Unix时间戳的毫秒数格式，如：1597026383085
        cTime             	String  	订单创建时间，Unix时间戳的毫秒数格式， 如 ：1597026383085
        '''
        return self.send_request(*_TradeEndpoints.get_order, **to_local(locals()))

    # GET / 获取未成交订单列表
    def get_orders_pending(self, instType: str = '', uly: str = '', instFamily: str = '', instId: str = '',
                           ordType: str = '', state: str = '', after: str = '', before: str = '', limit: str = '',
                           proxies={}, proxy_host: str = None):
        '''
        获取当前账户下所有未成交订单信息
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-get-order-list
        
        限速：60次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        instType          	String  	否       	产品类型SPOT：币币MARGIN：币币杠杆SWAP：永续合约FUTURES：交割合约OPTION：期权
        uly               	String  	否       	标的指数
        instFamily        	String  	否       	交易品种适用于交割/永续/期权
        instId            	String  	否       	产品ID，如BTC-USD-200927
        ordType           	String  	否       	订单类型market：市价单limit：限价单post_only：只做maker单fok：全部成交或立即取消ioc：立即成交并取消剩余optimal_limit_ioc：市价委托立即成交并取消剩余（仅适用交割、永续）mmp：做市商保护(仅适用于组合保证金账户模式下的期权订单)mmp_and_post_only：做市商保护且只做maker单(仅适用于组合保证金账户模式下的期权订单)
        state             	String  	否       	订单状态live：等待成交partially_filled：部分成交
        after             	String  	否       	请求此ID之前（更旧的数据）的分页内容，传的值为对应接口的ordId
        before            	String  	否       	请求此ID之后（更新的数据）的分页内容，传的值为对应接口的ordId
        limit             	String  	否       	返回结果的数量，最大为100，默认100条
        返回参数:
        Parameter         	Type    	Description
        instType          	String  	产品类型
        instId            	String  	产品ID
        tgtCcy            	String  	币币市价单委托数量sz的单位base_ccy: 交易货币 ；quote_ccy：计价货币仅适用于币币市价订单默认买单为quote_ccy，卖单为base_ccy
        ccy               	String  	保证金币种，仅适用于单币种保证金模式下的全仓币币杠杆订单
        ordId             	String  	订单ID
        clOrdId           	String  	客户自定义订单ID
        tag               	String  	订单标签
        px                	String  	委托价格，对于期权，以币(如BTC, ETH)为单位
        pxUsd             	String  	期权价格，以USD为单位仅适用于期权，其他业务线返回空字符串""
        pxVol             	String  	期权订单的隐含波动率仅适用于期权，其他业务线返回空字符串""
        pxType            	String  	期权的价格类型px：代表按价格下单，单位为币 (请求参数 px 的数值单位是BTC或ETH)pxVol：代表按pxVol下单pxUsd：代表按照pxUsd下单，单位为USD (请求参数px 的数值单位是USD)
        sz                	String  	委托数量
        pnl               	String  	收益，适用于有成交的平仓订单，其他情况均为0
        ordType           	String  	订单类型market：市价单limit：限价单post_only：只做maker单fok：全部成交或立即取消ioc：立即成交并取消剩余optimal_limit_ioc：市价委托立即成交并取消剩余（仅适用交割、永续）mmp：做市商保护(仅适用于组合保证金账户模式下的期权订单)mmp_and_post_only：做市商保护且只做maker单(仅适用于组合保证金账户模式下的期权订单)
        side              	String  	订单方向
        posSide           	String  	持仓方向
        tdMode            	String  	交易模式
        accFillSz         	String  	累计成交数量
        fillPx            	String  	最新成交价格。如果还没成交，系统返回""。
        tradeId           	String  	最新成交ID
        fillSz            	String  	最新成交数量
        fillTime          	String  	最新成交时间
        avgPx             	String  	成交均价。如果还没成交，系统返回0。
        state             	String  	订单状态live：等待成交partially_filled：部分成交
        lever             	String  	杠杆倍数，0.01到125之间的数值，仅适用于币币杠杆/交割/永续
        attachAlgoClOrdId 	String  	下单附带止盈止损时，客户自定义的策略订单ID
        tpTriggerPx       	String  	止盈触发价
        tpTriggerPxType   	String  	止盈触发价类型last：最新价格index：指数价格mark：标记价格
        slTriggerPx       	String  	止损触发价
        slTriggerPxType   	String  	止损触发价类型last：最新价格index：指数价格mark：标记价格
        slOrdPx           	String  	止损委托价
        tpOrdPx           	String  	止盈委托价
        stpId             	String  	自成交保护ID如果自成交保护不适用则返回""
        stpMode           	String  	自成交保护模式如果自成交保护不适用则返回""
        feeCcy            	String  	交易手续费币种
        fee               	String  	手续费与返佣对于币币和杠杆，为订单交易累计的手续费，平台向用户收取的交易手续费，为负数。如： -0.01对于交割、永续和期权，为订单交易累计的手续费和返佣
        rebateCcy         	String  	返佣金币种
        source            	String  	订单来源6：计划委托策略触发后的生成的普通单7：止盈止损策略触发后的生成的普通单13：策略委托单触发后的生成的普通单24：移动止盈止损策略触发后的生成的普通单
        rebate            	String  	返佣金额，仅适用于币币和杠杆，平台向达到指定lv交易等级的用户支付的挂单奖励（返佣），如果没有返佣金，该字段为“”。手续费返佣为正数，如：0.01
        category          	String  	订单种类normal： 普通委托
        reduceOnly        	String  	是否只减仓，true或false
        quickMgnType      	String  	一键借币类型，仅适用于杠杆逐仓的一键借币模式manual：手动，auto_borrow： 自动借币，auto_repay： 自动还币
        algoClOrdId       	String  	客户自定义策略订单ID。策略订单触发，且策略单有algoClOrdId是有值，否则为"",
        algoId            	String  	策略委托单ID，策略订单触发时有值，否则为""
        uTime             	String  	订单状态更新时间，Unix时间戳的毫秒数格式，如1597026383085
        cTime             	String  	订单创建时间，Unix时间戳的毫秒数格式，如1597026383085
        '''
        return self.send_request(*_TradeEndpoints.get_orders_pending, **to_local(locals()))

    # GET / 获取历史订单记录（近七天）
    def get_orders_history(self, instType: str, uly: str = '', instFamily: str = '', instId: str = '',
                           ordType: str = '', state: str = '', category: str = '', after: str = '', before: str = '',
                           begin: str = '', end: str = '', limit: str = '', proxies={}, proxy_host: str = None):
        '''
        获取最近7天挂单，且完全成交的订单数据，包括7天以前挂单，但近7天才成交的订单数据。按照订单创建时间倒序排序。
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-get-order-history-last-7-days
        
        限速：40次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        instType          	String  	是       	产品类型SPOT：币币MARGIN：币币杠杆SWAP：永续合约FUTURES：交割合约OPTION：期权
        uly               	String  	否       	标的指数
        instFamily        	String  	否       	交易品种适用于交割/永续/期权
        instId            	String  	否       	产品ID，如BTC-USD-190927
        ordType           	String  	否       	订单类型market：市价单limit：限价单post_only：只做maker单fok：全部成交或立即取消ioc：立即成交并取消剩余optimal_limit_ioc：市价委托立即成交并取消剩余（仅适用交割、永续）mmp：做市商保护(仅适用于组合保证金账户模式下的期权订单)mmp_and_post_only：做市商保护且只做maker单(仅适用于组合保证金账户模式下的期权订单)
        state             	String  	否       	订单状态canceled：撤单成功filled：完全成交mmp_canceled：做市商保护机制导致的自动撤单
        category          	String  	否       	订单种类twap：TWAP自动换币adl：ADL自动减仓full_liquidation：强制平仓partial_liquidation：强制减仓delivery：交割ddh：对冲减仓类型订单
        after             	String  	否       	请求此ID之前（更旧的数据）的分页内容，传的值为对应接口的ordId
        before            	String  	否       	请求此ID之后（更新的数据）的分页内容，传的值为对应接口的ordId
        begin             	String  	否       	筛选的开始时间戳，Unix 时间戳为毫秒数格式，如 1597026383085
        end               	String  	否       	筛选的结束时间戳，Unix 时间戳为毫秒数格式，如 1597027383085
        limit             	String  	否       	返回结果的数量，最大为100，默认100条
        返回参数:
        Parameter         	Type    	Description
        instType          	String  	产品类型
        instId            	String  	产品ID
        tgtCcy            	String  	币币市价单委托数量sz的单位base_ccy: 交易货币 ；quote_ccy：计价货币仅适用于币币市价订单默认买单为quote_ccy，卖单为base_ccy
        ccy               	String  	保证金币种，仅适用于单币种保证金模式下的全仓币币杠杆订单
        ordId             	String  	订单ID
        clOrdId           	String  	客户自定义订单ID
        tag               	String  	订单标签
        px                	String  	委托价格，对于期权，以币(如BTC, ETH)为单位
        pxUsd             	String  	期权价格，以USD为单位仅适用于期权，其他业务线返回空字符串""
        pxVol             	String  	期权订单的隐含波动率仅适用于期权，其他业务线返回空字符串""
        pxType            	String  	期权的价格类型px：代表按价格下单，单位为币 (请求参数 px 的数值单位是BTC或ETH)pxVol：代表按pxVol下单pxUsd：代表按照pxUsd下单，单位为USD (请求参数px 的数值单位是USD)
        sz                	String  	委托数量
        ordType           	String  	订单类型market：市价单limit：限价单post_only：只做maker单fok：全部成交或立即取消ioc：立即成交并取消剩余optimal_limit_ioc：市价委托立即成交并取消剩余（仅适用交割、永续）mmp：做市商保护(仅适用于组合保证金账户模式下的期权订单)mmp_and_post_only：做市商保护且只做maker单(仅适用于组合保证金账户模式下的期权订单)
        side              	String  	订单方向
        posSide           	String  	持仓方向
        tdMode            	String  	交易模式
        accFillSz         	String  	累计成交数量
        fillPx            	String  	最新成交价格，如果成交数量为0，该字段为""
        tradeId           	String  	最新成交ID
        fillSz            	String  	最新成交数量
        fillTime          	String  	最新成交时间
        avgPx             	String  	成交均价，如果成交数量为0，该字段也为""
        state             	String  	订单状态canceled：撤单成功filled：完全成交mmp_canceled：做市商保护机制导致的自动撤单
        lever             	String  	杠杆倍数，0.01到125之间的数值，仅适用于币币杠杆/交割/永续
        attachAlgoClOrdId 	String  	下单附带止盈止损时，客户自定义的策略订单ID
        tpTriggerPx       	String  	止盈触发价
        tpTriggerPxType   	String  	止盈触发价类型last：最新价格index：指数价格mark：标记价格
        tpOrdPx           	String  	止盈委托价
        slTriggerPx       	String  	止损触发价
        slTriggerPxType   	String  	止损触发价类型last：最新价格index：指数价格mark：标记价格
        slOrdPx           	String  	止损委托价
        stpId             	String  	自成交保护ID如果自成交保护不适用则返回""
        stpMode           	String  	自成交保护模式如果自成交保护不适用则返回""
        feeCcy            	String  	交易手续费币种
        fee               	String  	手续费与返佣对于币币和杠杆，为订单交易累计的手续费，平台向用户收取的交易手续费，为负数。如： -0.01对于交割、永续和期权，为订单交易累计的手续费和返佣
        rebateCcy         	String  	返佣金币种
        source            	String  	订单来源6：计划委托策略触发后的生成的普通单7：止盈止损策略触发后的生成的普通单13：策略委托单触发后的生成的普通单24：移动止盈止损策略触发后的生成的普通单
        rebate            	String  	返佣金额，仅适用于币币和杠杆，平台向达到指定lv交易等级的用户支付的挂单奖励（返佣），如果没有返佣金，该字段为“”。手续费返佣为正数，如：0.01
        pnl               	String  	收益，适用于有成交的平仓订单，其他情况均为0
        category          	String  	订单种类normal：普通委托twap：TWAP自动换币adl：ADL自动减仓full_liquidation：强制平仓partial_liquidation：强制减仓delivery：交割ddh：对冲减仓类型订单
        reduceOnly        	String  	是否只减仓，true或false
        cancelSource      	String  	订单取消来源的原因枚举值代码
        cancelSourceReason	String  	订单取消来源的对应具体原因
        algoClOrdId       	String  	客户自定义策略订单ID。策略订单触发，且策略单有algoClOrdId时有值，否则为"",
        algoId            	String  	策略委托单ID，策略订单触发时有值，否则为""
        uTime             	String  	订单状态更新时间，Unix时间戳的毫秒数格式，如1597026383085
        cTime             	String  	订单创建时间，Unix时间戳的毫秒数格式，如1597026383085
        '''
        return self.send_request(*_TradeEndpoints.get_orders_history, **to_local(locals()))

    # GET / 获取历史订单记录（近三个月）
    def get_orders_history_archive(self, instType: str, uly: str = '', instFamily: str = '', instId: str = '',
                                   ordType: str = '', state: str = '', category: str = '', after: str = '',
                                   before: str = '', begin: str = '', end: str = '', limit: str = '', proxies={},
                                   proxy_host: str = None):
        '''
        获取最近3个月挂单，且完全成交的订单数据，包括3个月以前挂单，但近3个月才成交的订单数据。按照订单创建时间倒序排序。
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-get-order-history-last-3-months
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        instType          	String  	是       	产品类型SPOT：币币MARGIN：币币杠杆SWAP：永续合约FUTURES：交割合约OPTION：期权
        uly               	String  	否       	标的指数
        instFamily        	String  	否       	交易品种适用于交割/永续/期权
        instId            	String  	否       	产品ID，如BTC-USD-200927
        ordType           	String  	否       	订单类型market：市价单limit：限价单post_only：只做maker单fok：全部成交或立即取消ioc：立即成交并取消剩余optimal_limit_ioc：市价委托立即成交并取消剩余（仅适用交割、永续）mmp：做市商保护(仅适用于组合保证金账户模式下的期权订单)mmp_and_post_only：做市商保护且只做maker单(仅适用于组合保证金账户模式下的期权订单)
        state             	String  	否       	订单状态canceled：撤单成功filled：完全成交mmp_canceled：做市商保护机制导致的自动撤单
        category          	String  	否       	订单种类twap：TWAP自动换币adl：ADL自动减仓full_liquidation：强制平仓partial_liquidation：强制减仓delivery：交割ddh：对冲减仓类型订单
        after             	String  	否       	请求此ID之前（更旧的数据）的分页内容，传的值为对应接口的ordId
        before            	String  	否       	请求此ID之后（更新的数据）的分页内容，传的值为对应接口的ordId
        begin             	String  	否       	筛选的开始时间戳，Unix 时间戳为毫秒数格式，如 1597026383085
        end               	String  	否       	筛选的结束时间戳，Unix 时间戳为毫秒数格式，如 1597027383085
        limit             	String  	否       	返回结果的数量，最大为100，默认100条
        返回参数:
        Parameter         	Type    	Description
        instType          	String  	产品类型
        instId            	String  	产品ID
        tgtCcy            	String  	币币市价单委托数量sz的单位base_ccy: 交易货币 ；quote_ccy：计价货币仅适用于币币市价订单默认买单为quote_ccy，卖单为base_ccy
        ccy               	String  	保证金币种，仅适用于单币种保证金模式下的全仓币币杠杆订单
        ordId             	String  	订单ID
        clOrdId           	String  	客户自定义订单ID
        tag               	String  	订单标签
        px                	String  	委托价格，对于期权，以币(如BTC, ETH)为单位
        pxUsd             	String  	期权价格，以USD为单位仅适用于期权，其他业务线返回空字符串""
        pxVol             	String  	期权订单的隐含波动率仅适用于期权，其他业务线返回空字符串""
        pxType            	String  	期权的价格类型px：代表按价格下单，单位为币 (请求参数 px 的数值单位是BTC或ETH)pxVol：代表按pxVol下单pxUsd：代表按照pxUsd下单，单位为USD (请求参数px 的数值单位是USD)
        sz                	String  	委托数量
        ordType           	String  	订单类型market：市价单limit：限价单post_only：只做maker单fok：全部成交或立即取消ioc：立即成交并取消剩余optimal_limit_ioc：市价委托立即成交并取消剩余（仅适用交割、永续）mmp：做市商保护(仅适用于组合保证金账户模式下的期权订单)mmp_and_post_only：做市商保护且只做maker单(仅适用于组合保证金账户模式下的期权订单)
        side              	String  	订单方向
        posSide           	String  	持仓方向
        tdMode            	String  	交易模式
        accFillSz         	String  	累计成交数量
        fillPx            	String  	最新成交价格，如果成交数量为0，该字段为""
        tradeId           	String  	最新成交ID
        fillSz            	String  	最新成交数量
        fillTime          	String  	最新成交时间
        avgPx             	String  	成交均价，如果成交数量为0，该字段也为""
        state             	String  	订单状态canceled：撤单成功filled：完全成交mmp_canceled：做市商保护机制导致的自动撤单
        lever             	String  	杠杆倍数，0.01到125之间的数值，仅适用于币币杠杆/交割/永续
        attachAlgoClOrdId 	String  	下单附带止盈止损时，客户自定义的策略订单ID
        tpTriggerPx       	String  	止盈触发价
        tpTriggerPxType   	String  	止盈触发价类型last：最新价格index：指数价格mark：标记价格
        tpOrdPx           	String  	止盈委托价
        slTriggerPx       	String  	止损触发价
        slTriggerPxType   	String  	止损触发价类型last：最新价格index：指数价格mark：标记价格
        slOrdPx           	String  	止损委托价
        stpId             	String  	自成交保护ID如果自成交保护不适用则返回""
        stpMode           	String  	自成交保护模式如果自成交保护不适用则返回""
        feeCcy            	String  	交易手续费币种
        fee               	String  	手续费与返佣对于币币和杠杆，为订单交易累计的手续费，平台向用户收取的交易手续费，为负数。如： -0.01对于交割、永续和期权，为订单交易累计的手续费和返佣
        rebateCcy         	String  	返佣金币种
        rebate            	String  	返佣金额，仅适用于币币和杠杆，平台向达到指定lv交易等级的用户支付的挂单奖励（返佣），如果没有返佣金，该字段为“”。手续费返佣为正数，如：0.01
        pnl               	String  	收益，适用于有成交的平仓订单，其他情况均为0
        source            	String  	订单来源6：计划委托策略触发后的生成的普通单7：止盈止损策略触发后的生成的普通单13：策略委托单触发后的生成的普通单24：移动止盈止损策略触发后的生成的普通单
        category          	String  	订单种类normal：普通委托twap：TWAP自动换币adl：ADL自动减仓full_liquidation：强制平仓partial_liquidation：强制减仓delivery：交割ddh：对冲减仓类型订单
        reduceOnly        	String  	是否只减仓，true或false
        cancelSource      	String  	订单取消来源的原因枚举值代码
        cancelSourceReason	String  	订单取消来源的对应具体原因
        algoClOrdId       	String  	客户自定义策略订单ID。策略订单触发，且策略单有algoClOrdId是有值，否则为"",
        algoId            	String  	策略委托单ID，策略订单触发时有值，否则为""
        uTime             	String  	订单状态更新时间，Unix时间戳的毫秒数格式，如1597026383085
        cTime             	String  	订单创建时间，Unix时间戳的毫秒数格式，如1597026383085
        '''
        return self.send_request(*_TradeEndpoints.get_orders_history_archive, **to_local(locals()))

    # GET / 获取成交明细（近三天）
    def get_fills(self, instType: str = '', uly: str = '', instFamily: str = '', instId: str = '', ordId: str = '',
                  after: str = '', before: str = '', begin: str = '', end: str = '', limit: str = '', proxies={},
                  proxy_host: str = None):
        '''
        获取近3天的订单成交明细信息
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-get-transaction-details-last-3-days
        
        限速：60次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        instType          	String  	否       	产品类型SPOT：币币MARGIN：币币杠杆SWAP：永续合约FUTURES：交割合约OPTION：期权
        uly               	String  	否       	标的指数
        instFamily        	String  	否       	交易品种适用于交割/永续/期权
        instId            	String  	否       	产品 ID，如BTC-USD-190927
        ordId             	String  	否       	订单 ID
        after             	String  	否       	请求此 ID 之前（更旧的数据）的分页内容，传的值为对应接口的billId
        before            	String  	否       	请求此 ID 之后（更新的数据）的分页内容，传的值为对应接口的billId
        begin             	String  	否       	筛选的开始时间戳，Unix 时间戳为毫秒数格式，如 1597026383085
        end               	String  	否       	筛选的结束时间戳，Unix 时间戳为毫秒数格式，如 1597027383085
        limit             	String  	否       	返回结果的数量，最大为100，默认100条
        返回参数:
        Parameter         	Type    	Description
        instType          	String  	产品类型
        instId            	String  	产品 ID
        tradeId           	String  	最新成交 ID
        ordId             	String  	订单 ID
        clOrdId           	String  	用户自定义订单ID
        billId            	String  	账单 ID
        tag               	String  	订单标签
        fillPx            	String  	最新成交价格
        fillSz            	String  	最新成交数量
        fillIdxPx         	String  	交易执行时的指数价格对于交叉现货币对，返回 baseCcy-USDT 的指数价格。 例如LTC-ETH，该字段返回LTC-USDT的指数价格。
        fillPnl           	String  	最新成交收益，适用于有成交的平仓订单。其他情况均为0。
        fillPxVol         	String  	成交时的隐含波动率，仅适用于期权，其他业务线返回空字符串""
        fillPxUsd         	String  	成交时的期权价格，以USD为单位，仅适用于期权，其他业务线返回空字符串""
        fillMarkVol       	String  	成交时的标记波动率，仅适用于期权，其他业务线返回空字符串""
        fillFwdPx         	String  	成交时的远期价格，仅适用于期权，其他业务线返回空字符串""
        fillMarkPx        	String  	成交时的标记价格，仅适用于交割/永续/期权
        side              	String  	订单方向buy：买sell：卖
        posSide           	String  	持仓方向long：多short：空 买卖模式返回net
        execType          	String  	流动性方向T：takerM：maker不适用于系统订单比如强平和ADL
        feeCcy            	String  	交易手续费币种或者返佣金币种
        fee               	String  	手续费金额或者返佣金额，手续费扣除为‘负数’，如-0.01；手续费返佣为‘正数’，如 0.01
        ts                	String  	成交明细产生时间，Unix时间戳的毫秒数格式，如1597026383085
        fillTime          	String  	成交时间，与订单频道的fillTime相同
        '''
        return self.send_request(*_TradeEndpoints.get_fills, **to_local(locals()))

    # GET / 获取成交明细（近三个月）
    def get_fills_history(self, instType: str, uly: str = '', instFamily: str = '', instId: str = '', ordId: str = '',
                          after: str = '', before: str = '', begin: str = '', end: str = '', limit: str = '',
                          proxies={}, proxy_host: str = None):
        '''
        获取近3个月订单成交明细信息
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-get-transaction-details-last-3-months
        
        限速：10 次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        instType          	String  	是       	产品类型SPOT：币币MARGIN：币币杠杆SWAP：永续合约FUTURES：交割合约OPTION：期权
        uly               	String  	否       	标的指数
        instFamily        	String  	否       	交易品种适用于交割/永续/期权
        instId            	String  	否       	产品 ID，如BTC-USD-190927
        ordId             	String  	否       	订单 ID
        after             	String  	否       	请求此 ID 之前（更旧的数据）的分页内容，传的值为对应接口的billId
        before            	String  	否       	请求此 ID 之后（更新的数据）的分页内容，传的值为对应接口的billId
        begin             	String  	否       	筛选的开始时间戳，Unix 时间戳为毫秒数格式，如 1597026383085
        end               	String  	否       	筛选的结束时间戳，Unix 时间戳为毫秒数格式，如 1597027383085
        limit             	String  	否       	返回结果的数量，最大为100，默认100条
        返回参数:
        Parameter         	Type    	Description
        instType          	String  	产品类型
        instId            	String  	产品 ID
        tradeId           	String  	最新成交 ID
        ordId             	String  	订单 ID
        clOrdId           	String  	用户自定义订单ID
        billId            	String  	账单 ID
        tag               	String  	订单标签
        fillPx            	String  	最新成交价格
        fillSz            	String  	最新成交数量
        fillIdxPx         	String  	交易执行时的指数价格对于交叉现货币对，返回 baseCcy-USDT 的指数价格。 例如LTC-ETH，该字段返回LTC-USDT的指数价格。
        fillPnl           	String  	最新成交收益，适用于有成交的平仓订单。其他情况均为0。
        fillPxVol         	String  	成交时的隐含波动率，仅适用于期权，其他业务线返回空字符串""
        fillPxUsd         	String  	成交时的期权价格，以USD为单位，仅适用于期权，其他业务线返回空字符串""
        fillMarkVol       	String  	成交时的标记波动率，仅适用于期权，其他业务线返回空字符串""
        fillFwdPx         	String  	成交时的远期价格，仅适用于期权，其他业务线返回空字符串""
        fillMarkPx        	String  	成交时的标记价格，仅适用于交割/永续/期权
        side              	String  	订单方向buy：买sell：卖
        posSide           	String  	持仓方向long：多short：空 买卖模式返回net
        execType          	String  	流动性方向T：takerM：maker不适用于系统订单比如强平和ADL
        feeCcy            	String  	交易手续费币种或者返佣金币种
        fee               	String  	手续费金额或者返佣金额 ，手续费扣除 为 ‘负数’，如 -0.01 ； 手续费返佣 为 ‘正数’，如 0.01
        ts                	String  	成交明细产生时间，Unix时间戳的毫秒数格式，如1597026383085
        fillTime          	String  	成交时间，与订单频道的fillTime相同
        '''
        return self.send_request(*_TradeEndpoints.get_fills_history, **to_local(locals()))

    # POST / 申请成交明细（近两年）
    def set_fills_archive(self, year: str, quarter: str, proxies={}, proxy_host: str = None):
        '''
        申请近2年的历史成交成交明细信息，不包括最近三个月。
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-post-transaction-details-in-the-past-2-years
        
        限速：5 次/天
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        year              	String  	是       	4位数字的年份，如2023
        quarter           	String  	是       	季度，有效值Q1Q2Q3Q4
        返回参数:
        Parameter         	Type    	Description
        ts                	String  	下载链接生成时间，Unix时间戳的毫秒数格式 ，如1597026383085
        '''
        return self.send_request(*_TradeEndpoints.set_fills_archive, **to_local(locals()))

    # GET / 获取成交明细（近两年）
    def get_fills_archive(self, year: str, quarter: str, proxies={}, proxy_host: str = None):
        '''
        获取近2年的历史成交明细信息
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-get-transaction-details-in-the-past-2-years
        
        限速：10 次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        year              	String  	是       	4位数字的年份，如2023
        quarter           	String  	是       	季度，有效值Q1Q2Q3Q4
        返回参数:
        Parameter         	Type    	Description
        fileHref          	String  	文件链接
        ts                	String  	下载链接生成时间，Unix时间戳的毫秒数格式 ，如1597026383085
        state             	String  	下载链接状态finished：已生成ongoing：进行中
        '''
        return self.send_request(*_TradeEndpoints.get_fills_archive, **to_local(locals()))

    # GET / 获取一键兑换主流币币种列表
    def get_easy_convert_currency_list(self, proxies={}, proxy_host: str = None):
        '''
        获取小币一键兑换主流币币种列表。仅可兑换余额在 $10 以下小币币种。
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-get-easy-convert-currency-list
        
        限速：1次/2s
        限速规则：UserID
    
        
        返回参数:
        Parameter         	Type    	Description
        fromData          	Array   	当前拥有并可兑换的小币币种列表信息
        > fromCcy         	String  	可兑换币种
        > fromAmt         	String  	可兑换币种数量
        toCcy             	Array   	可转换成的主流币币种列表
        '''
        return self.send_request(*_TradeEndpoints.get_easy_convert_currency_list, **to_local(locals()))

    # POST / 一键兑换主流币交易
    def set_easy_convert(self, fromCcy: list, toCcy: str, proxies={}, proxy_host: str = None):
        '''
        进行小币一键兑换主流币交易。仅可兑换余额在 $10 以下小币币种。
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-post-place-easy-convert
        
        限速：1次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        fromCcy           	Array   	是       	小币支付币种单次最多同时选择5个币种，如有多个币种则用逗号隔开
        toCcy             	String  	是       	兑换的主流币只选择一个币种，且不能和小币支付币种重复
        返回参数:
        Parameter         	Type    	Description
        status            	String  	当前兑换进度/状态running: 进行中filled: 已完成failed: 失败
        fromCcy           	String  	小币支付币种
        toCcy             	String  	兑换的主流币
        fillFromSz        	String  	小币偿还币种支付数量
        fillToSz          	String  	兑换的主流币成交数量
        uTime             	String  	交易时间戳，Unix时间戳为毫秒数格式，如 1597026383085
        '''
        return self.send_request(*_TradeEndpoints.set_easy_convert, **to_local(locals()))

    # GET / 获取一键兑换主流币历史记录
    def get_easy_convert_history(self, after: str = '', before: str = '', limit: str = '', proxies={},
                                 proxy_host: str = None):
        '''
        查询一键兑换主流币的历史记录与进度状态。
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-get-easy-convert-history
        
        限速：1次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        after             	String  	否       	查询在此之前的内容，值为时间戳，Unix时间戳为毫秒数格式，如1597026383085
        before            	String  	否       	查询在此之后的内容，值为时间戳，Unix时间戳为毫秒数格式，如1597026383085
        limit             	String  	否       	返回的结果集数量，默认为100，最大为100
        返回参数:
        Parameter         	Type    	Description
        fromCcy           	String  	小币支付币种
        fromSz            	String  	对应的小币支付数量
        toCcy             	String  	兑换到的主流币
        toSz              	String  	兑换到的主流币数量
        status            	String  	当前兑换进度/状态running: 进行中filled: 已完成failed: 失败
        uTime             	String  	交易时间戳，Unix时间戳为毫秒数格式，如 1597026383085
        '''
        return self.send_request(*_TradeEndpoints.get_easy_convert_history, **to_local(locals()))

    # GET / 获取一键还债币种列表
    def get_one_click_repay_currency_list(self, debtType: str = '', proxies={}, proxy_host: str = None):
        '''
        查询一键还债币种列表。负债币种包括全仓负债和逐仓负债。
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-get-one-click-repay-currency-list
        
        限速：1次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        debtType          	String  	否       	负债类型cross: 全仓负债isolated: 逐仓负债
        返回参数:
        Parameter         	Type    	Description
        debtData          	Array   	负债币种信息
        > debtCcy         	String  	负债币种
        > debtAmt         	String  	可负债币种数量包括本金和利息
        debtType          	String  	负债类型cross: 全仓负债isolated: 逐仓负债
        repayData         	Array   	偿还币种信息
        > repayCcy        	String  	可偿还负债的币种
        > repayAmt        	String  	可偿还负债的币种可用资产数量
        '''
        return self.send_request(*_TradeEndpoints.get_one_click_repay_currency_list, **to_local(locals()))

    # POST / 一键还债交易
    def set_one_click_repay(self, debtCcy: list, repayCcy: str, proxies={}, proxy_host: str = None):
        '''
        交易一键偿还小额全仓债务。不支持逐仓负债的偿还。根据资金和交易账户的剩余可用余额为最大偿还数量。
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-post-trade-one-click-repay
        
        限速：1次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        debtCcy           	Array   	是       	负债币种单次最多同时选择5个币种，如有多个币种则用逗号隔开
        repayCcy          	String  	是       	偿还币种只选择一个币种，且不能和负债币种重复
        返回参数:
        Parameter         	Type    	Description
        status            	String  	当前还债进度/状态running: 进行中filled: 已完成failed: 失败
        debtCcy           	String  	负债币种
        repayCcy          	String  	偿还币种
        fillDebtSz        	String  	负债币种成交数量
        fillRepaySz       	String  	偿还币种成交数量
        uTime             	String  	交易时间戳，Unix时间戳为毫秒数格式，如 1597026383085
        '''
        return self.send_request(*_TradeEndpoints.set_one_click_repay, **to_local(locals()))

    # GET / 获取一键还债历史记录
    def get_one_click_repay_history(self, after: str = '', before: str = '', limit: str = '', proxies={},
                                    proxy_host: str = None):
        '''
        查询一键还债的历史记录与进度状态。
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-get-one-click-repay-history
        
        限速：1次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        after             	String  	否       	查询在此之前的内容，值为时间戳，Unix时间戳为毫秒数格式，如1597026383085
        before            	String  	否       	查询在此之后的内容，值为时间戳，Unix时间戳为毫秒数格式，如1597026383085
        limit             	String  	否       	返回的结果集数量，默认为100，最大为100
        返回参数:
        Parameter         	Type    	Description
        debtCcy           	String  	负债币种
        debtSz            	String  	对应的负债币种成交数量
        repayCcy          	String  	偿还币种
        repaySz           	String  	偿还币种实际支付数量
        status            	String  	当前还债进度/状态running: 进行中filled: 已完成failed: 失败
        uTime             	String  	交易时间戳，Unix时间戳为毫秒数格式，如 1597026383085
        '''
        return self.send_request(*_TradeEndpoints.get_one_click_repay_history, **to_local(locals()))

    # POST / 撤销 MMP 订单
    def set_mass_cancel(self, instType: str, instFamily: str, proxies={}, proxy_host: str = None):
        '''
        撤销同一交易品种下用户所有的 MMP 挂单<!- 1-2-3 -->仅适用于组合保证金账户模式下的期权订单，且有 MMP 权限。
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-post-mass-cancel-order
        
        限速：5次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        instType          	String  	是       	交易产品类型OPTION:期权
        instFamily        	String  	是       	交易品种
        返回参数:
        Parameter         	Type    	Description
        result            	Boolean 	撤单结果true：全部撤单成功false：全部撤单失败
        '''
        return self.send_request(*_TradeEndpoints.set_mass_cancel, **to_local(locals()))

    # POST / 倒计时全部撤单
    def set_cancel_all_after(self, timeOut: str, proxies={}, proxy_host: str = None):
        '''
        在倒计时结束后，取消所有 MMP 的挂单。<!- 1-2-3 -->仅适用于组合保证金账户模式下的期权订单，且有 MMP 权限。
        https://www.okx.com/docs-v5/zh/#order-book-trading-trade-post-cancel-all-after
        
        限速：1次/1s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        timeOut           	String  	是       	取消 MMP 挂单的倒计时，单位为秒。取值范围为 0, [10, 120]0 代表不使用该功能。
        返回参数:
        Parameter         	Type    	Description
        triggerTime       	String  	触发撤单的时间.triggerTime=0 代表未使用该功能。
        ts                	String  	请求时间
        '''
        return self.send_request(*_TradeEndpoints.set_cancel_all_after, **to_local(locals()))



'''
网格交易
https://www.okx.com/docs-v5/zh/#order-book-trading-grid-trading
'''

from paux.param import to_local
from okx.api._client import Client


class _GridTradeEndpoints():
    set_order_algo = ['/api/v5/tradingBot/grid/order-algo', 'POST']  # POST / 网格策略委托下单
    set_amend_order_algo = ['/api/v5/tradingBot/grid/amend-order-algo', 'POST']  # POST / 修改网格策略订单
    set_stop_order_algo = ['/api/v5/tradingBot/grid/stop-order-algo', 'POST']  # POST / 网格策略停止
    set_close_position = ['/api/v5/tradingBot/grid/close-position', 'POST']  # POST / 合约网格平仓
    set_cancel_close_order = ['/api/v5/tradingBot/grid/cancel-close-order', 'POST']  # POST / 撤销合约网格平仓单
    set_order_instant_trigger = ['/api/v5/tradingBot/grid/order-instant-trigger', 'POST']  # POST / 网格策略立即触发
    get_orders_algo_pending = ['/api/v5/tradingBot/grid/orders-algo-pending', 'GET']  # GET / 获取未完成网格策略委托单列表
    get_orders_algo_history = ['/api/v5/tradingBot/grid/orders-algo-history', 'GET']  # GET / 获取历史网格策略委托单列表
    get_orders_algo_details = ['/api/v5/tradingBot/grid/orders-algo-details', 'GET']  # GET / 获取网格策略委托订单详情
    get_sub_orders = ['/api/v5/tradingBot/grid/sub-orders', 'GET']  # GET / 获取网格策略委托子订单信息
    get_positions = ['/api/v5/tradingBot/grid/positions', 'GET']  # GET / 获取网格策略委托持仓
    set_withdraw_income = ['/api/v5/tradingBot/grid/withdraw-income', 'POST']  # POST / 现货/天地网格提取利润
    set_compute_margin_balance = ['/api/v5/tradingBot/grid/compute-margin-balance', 'POST']  # POST / 调整保证金计算
    set_margin_balance = ['/api/v5/tradingBot/grid/margin-balance', 'POST']  # POST / 调整保证金
    get_ai_param = ['/api/v5/tradingBot/grid/ai-param', 'GET']  # GET / 网格策略智能回测（公共）
    set_min_investment = ['/api/v5/tradingBot/grid/min-investment', 'POST']  # POST / 计算最小投资数量（公共）
    get_rsi_back_testing = ['/api/v5/tradingBot/public/rsi-back-testing', 'GET']  # GET / RSI回测（公共）


class GridTrade(Client):

    # POST / 网格策略委托下单
    def set_order_algo(self, instId: str, algoOrdType: str, maxPx: str, minPx: str, gridNum: str, runType: str = '',
                       tpTriggerPx: str = '', slTriggerPx: str = '', algoClOrdId: str = '', tag: str = '',
                       triggerParams: object = '', proxies={}, proxy_host: str = None):
        '''
        POST /api/v5/tradingBot/grid/order-algo
        https://www.okx.com/docs-v5/zh/#order-book-trading-grid-trading-post-place-grid-algo-order
        
        限速：20次/2s
        限速规则：UserID + Instrument ID
    
        请求参数:
        Parameter         	Type    	Required	Description

        instId            	String  	是       	产品ID，如BTC-USDT
        algoOrdType       	String  	是       	策略订单类型grid：现货网格委托contract_grid：合约网格委托moon_grid：天地网格委托
        maxPx             	String  	是       	区间最高价格
        minPx             	String  	是       	区间最低价格
        gridNum           	String  	是       	网格数量
        runType           	String  	否       	网格类型1：等差，2：等比默认为等差天地网格只支持2
        tpTriggerPx       	String  	否       	止盈触发价适用于现货网格/合约网格
        slTriggerPx       	String  	否       	止损触发价适用于现货网格/合约网格
        algoClOrdId       	String  	否       	用户自定义策略ID字母（区分大小写）与数字的组合，可以是纯字母、纯数字且长度要在1-32位之间。
        tag               	String  	否       	订单标签
        triggerParams     	Array of object	否       	信号触发参数适用于现货网格/合约网格
        > triggerAction   	String  	是       	触发行为start：网格启动stop：网格停止
        > triggerStrategy 	String  	是       	触发策略instant：立即触发price：价格触发rsi：rsi指标触发默认为instant
        > delaySeconds    	String  	否       	延迟触发时间，单位为秒，默认为0
        > timeframe       	String  	否       	K线种类3m,5m,15m,30m(m代表分钟)1H,4H(H代表小时)1D(D代表天)该字段只在triggerStrategy为rsi时有效
        > thold           	String  	否       	阈值取值[1,100]的整数该字段只在triggerStrategy为rsi时有效
        > triggerCond     	String  	否       	触发条件cross_up：上穿cross_down：下穿above：上方below：下方cross：交叉该字段只在triggerStrategy为rsi时有效
        > timePeriod      	String  	否       	周期14该字段只在triggerStrategy为rsi下有效
        > triggerPx       	String  	否       	触发价格该字段只在triggerStrategy为price下有效
        > stopType        	String  	否       	策略停止类型现货/天地网格1：卖出交易币，2：不卖出交易币合约网格1：停止平仓，2：停止不平仓该字段只在triggerAction为stop时有效
        返回参数:
        Parameter         	Type    	Description
        algoId            	String  	策略订单ID
        algoClOrdId       	String  	用户自定义策略ID
        sCode             	String  	事件执行结果的code，0代表成功
        sMsg              	String  	事件执行失败时的msg
        tag               	String  	订单标签
        '''
        return self.send_request(*_GridTradeEndpoints.set_order_algo, **to_local(locals()))

    # POST / 修改网格策略订单
    def set_amend_order_algo(self, algoId: str, instId: str, slTriggerPx: str = '', tpTriggerPx: str = '',
                             triggerParams: object = '', proxies={}, proxy_host: str = None):
        '''
        只支持合约网格策略的修改
        https://www.okx.com/docs-v5/zh/#order-book-trading-grid-trading-post-amend-grid-algo-order
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        algoId            	String  	是       	策略订单ID
        instId            	String  	是       	产品ID，如BTC-USDT-SWAP
        slTriggerPx       	String  	可选      	新的止损触发价当值为""则代表取消止损触发价slTriggerPx、tpTriggerPx至少要传一个值
        tpTriggerPx       	String  	可选      	新的止盈触发价当值为""则代表取消止盈触发价
        triggerParams     	Array of object	否       	信号触发参数
        > triggerAction   	String  	是       	触发行为start：网格启动stop：网格停止
        > triggerStrategy 	String  	是       	触发策略instant：立即触发price：价格触发rsi：rsi指标触发
        > triggerPx       	String  	否       	触发价格该字段只在triggerStrategy为price下有效
        > stopType        	String  	否       	策略停止类型现货/天地网格1：卖出交易币，2：不卖出交易币合约网格1：停止平仓，2：停止不平仓该字段只在triggerAction为stop时有效
        返回参数:
        Parameter         	Type    	Description
        algoId            	String  	策略订单ID
        algoClOrdId       	String  	用户自定义策略ID
        sCode             	String  	事件执行结果的code，0代表成功
        sMsg              	String  	事件执行失败时的msg
        tag               	String  	订单标签
        '''
        return self.send_request(*_GridTradeEndpoints.set_amend_order_algo, **to_local(locals()))

    # POST / 网格策略停止
    def set_stop_order_algo(self, algoId: str, instId: str, algoOrdType: str, stopType: str, proxies={},
                            proxy_host: str = None):
        '''
        每次最多可以撤销10个网格策略。
        https://www.okx.com/docs-v5/zh/#order-book-trading-grid-trading-post-stop-grid-algo-order
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        algoId            	String  	是       	策略订单ID
        instId            	String  	是       	产品ID，如BTC-USDT
        algoOrdType       	String  	是       	策略订单类型grid：现货网格委托contract_grid：合约网格委托moon_grid：天地网格委托
        stopType          	String  	是       	网格策略停止类型现货网格/天地网格1：卖出交易币，2：不卖出交易币合约网格1：市价全平2：停止不平仓
        返回参数:
        Parameter         	Type    	Description
        algoId            	String  	策略订单ID
        algoClOrdId       	String  	用户自定义策略ID
        sCode             	String  	事件执行结果的code，0代表成功
        sMsg              	String  	事件执行失败时的msg
        tag               	String  	订单标签
        '''
        return self.send_request(*_GridTradeEndpoints.set_stop_order_algo, **to_local(locals()))

    # POST / 合约网格平仓
    def set_close_position(self, algoId: str, mktClose: bool, sz: str = '', px: str = '', proxies={},
                           proxy_host: str = None):
        '''
        只有处于已停止未平仓状态合约网格可使用该接口
        https://www.okx.com/docs-v5/zh/#order-book-trading-grid-trading-post-close-position-for-contract-grid
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        algoId            	String  	是       	策略订单ID
        mktClose          	Boolean 	是       	是否市价全平true：市价全平，false：部分平仓
        sz                	String  	可选      	平仓数量,单位为张部分平仓时必传
        px                	String  	可选      	平仓价格部分平仓时必传
        返回参数:
        Parameter         	Type    	Description
        algoId            	String  	策略订单ID
        ordId             	String  	平仓单ID市价全平时，该字段为""
        algoClOrdId       	String  	用户自定义策略ID
        tag               	String  	订单标签
        '''
        return self.send_request(*_GridTradeEndpoints.set_close_position, **to_local(locals()))

    # POST / 撤销合约网格平仓单
    def set_cancel_close_order(self, algoId: str, ordId: str, proxies={}, proxy_host: str = None):
        '''
        POST /api/v5/tradingBot/grid/cancel-close-order
        https://www.okx.com/docs-v5/zh/#order-book-trading-grid-trading-post-cancel-close-position-order-for-contract-grid
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        algoId            	String  	是       	策略订单ID
        ordId             	String  	是       	平仓单ID
        返回参数:
        Parameter         	Type    	Description
        algoId            	String  	策略订单ID
        ordId             	String  	平仓单ID
        algoClOrdId       	String  	用户自定义策略ID
        tag               	String  	订单标签
        '''
        return self.send_request(*_GridTradeEndpoints.set_cancel_close_order, **to_local(locals()))

    # POST / 网格策略立即触发
    def set_order_instant_trigger(self, algoId: str, proxies={}, proxy_host: str = None):
        '''
        POST /api/v5/tradingBot/grid/order-instant-trigger
        https://www.okx.com/docs-v5/zh/#order-book-trading-grid-trading-post-instant-trigger-grid-algo-order
        
        限速：20次/2s
        限速规则：UserID + Instrument ID
    
        请求参数:
        Parameter         	Type    	Required	Description

        algoId            	String  	是       	策略订单ID
        返回参数:
        Parameter         	Type    	Description
        algoId            	String  	策略订单ID
        algoClOrdId       	String  	用户自定义策略ID
        '''
        return self.send_request(*_GridTradeEndpoints.set_order_instant_trigger, **to_local(locals()))

    # GET / 获取未完成网格策略委托单列表
    def get_orders_algo_pending(self, algoOrdType: str, algoId: str = '', instId: str = '', instType: str = '',
                                after: str = '', before: str = '', limit: str = '', proxies={}, proxy_host: str = None):
        '''
        GET /api/v5/tradingBot/grid/orders-algo-pending
        https://www.okx.com/docs-v5/zh/#order-book-trading-grid-trading-get-grid-algo-order-list
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        algoOrdType       	String  	是       	策略订单类型grid：现货网格委托contract_grid：合约网格委托moon_grid：天地网格委托
        algoId            	String  	否       	策略订单ID
        instId            	String  	否       	产品ID，如BTC-USDT
        instType          	String  	否       	产品类型SPOT：币币MARGIN：杠杆FUTURES：交割合约SWAP：永续合约
        after             	String  	否       	请求此ID之前（更旧的数据）的分页内容，传的值为对应接口的algoId
        before            	String  	否       	请求此ID之后（更新的数据）的分页内容，传的值为对应接口的algoId
        limit             	String  	否       	返回结果的数量，最大为100，默认100条
        返回参数:
        Parameter         	Type    	Description
        algoId            	String  	策略订单ID
        algoClOrdId       	String  	用户自定义策略ID
        instType          	String  	产品类型
        instId            	String  	产品ID
        cTime             	String  	策略订单创建时间，Unix时间戳的毫秒数格式，如1597026383085
        uTime             	String  	策略订单更新时间，Unix时间戳的毫秒数格式，如1597026383085
        algoOrdType       	String  	策略订单类型grid：现货网格委托contract_grid：合约网格委托moon_grid：天地网格委托
        state             	String  	订单状态starting：启动中running：运行中stopping：终止中no_close_position：已停止未平仓（仅适用于合约网格）
        rebateTrans       	Array of object	返佣划转信息
        > rebate          	String  	返佣数量
        > rebateCcy       	String  	返佣币种
        triggerParams     	Array of object	信号触发参数
        > triggerAction   	String  	触发行为start：网格启动stop：网格停止
        > triggerStrategy 	String  	触发策略instant：立即触发price：价格触发rsi：rsi指标触发
        > delaySeconds    	String  	延迟触发时间，单位为秒
        > triggerTime     	String  	triggerAction实际触发时间，Unix时间戳的毫秒数格式, 如1597026383085
        > triggerType     	String  	triggerAction的实际触发类型manual：手动触发auto: 自动触发
        > timeframe       	String  	K线种类3m,5m,15m,30m(m代表分钟)1H,4H(H代表小时)1D(D代表天)该字段只在triggerStrategy为rsi时有效
        > thold           	String  	阈值取值[1,100]的整数该字段只在triggerStrategy为rsi时有效
        > triggerCond     	String  	触发条件cross_up：上穿cross_down：下穿above：上方below：下方cross：交叉该字段只在triggerStrategy为rsi时有效
        > timePeriod      	String  	周期14该字段只在triggerStrategy为rsi下有效
        > triggerPx       	String  	触发价格该字段只在triggerStrategy为price下有效
        > stopType        	String  	策略停止类型现货/天地网格1：卖出交易币，2：不卖出交易币合约网格1：停止平仓，2：停止不平仓该字段只在triggerAction为stop时有效
        maxPx             	String  	区间最高价格
        minPx             	String  	区间最低价格
        gridNum           	String  	网格数量
        runType           	String  	网格类型1：等差，2：等比
        tpTriggerPx       	String  	止盈触发价
        slTriggerPx       	String  	止损触发价
        arbitrageNum      	String  	网格套利次数
        totalPnl          	String  	总收益
        pnlRatio          	String  	收益率
        investment        	String  	投入金额现货网格如果投入了交易币则折算为计价币
        gridProfit        	String  	网格利润
        floatProfit       	String  	浮动盈亏
        cancelType        	String  	网格策略停止原因0：无1：手动停止2：止盈停止3：止损停止4：风控停止5：交割停止6: 信号停止
        stopType          	String  	网格策略实际停止类型现货网格/天地网格1：卖出交易币，2：不卖出交易币合约网格1：停止平仓，2：停止不平仓
        quoteSz           	String  	计价币投入数量适用于现货网格/天地网格
        baseSz            	String  	交易币投入数量适用于现货网格
        direction         	String  	合约网格类型long：做多，short：做空，neutral：中性仅适用于合约网格
        basePos           	Boolean 	是否开底仓适用于合约网格
        sz                	String  	投入保证金，单位为USDT适用于合约网格
        lever             	String  	杠杆倍数适用于合约网格
        actualLever       	String  	实际杠杆倍数适用于合约网格
        liqPx             	String  	预估强平价格适用于合约网格
        uly               	String  	标的指数适用于合约网格
        instFamily        	String  	交易品种适用于交割/永续/期权，如BTC-USD适用于合约网格
        ordFrozen         	String  	挂单占用适用于合约网格
        availEq           	String  	可用保证金适用于合约网格
        tag               	String  	订单标签
        profitSharingRatio	String  	分润比例取值范围[0,0.3]如果是普通订单（既不是带单也不是跟单），该字段返回""
        copyType          	String  	分润订单类型0：普通订单1：普通跟单2：分润跟单3：带单
        '''
        return self.send_request(*_GridTradeEndpoints.get_orders_algo_pending, **to_local(locals()))

    # GET / 获取历史网格策略委托单列表
    def get_orders_algo_history(self, algoOrdType: str, algoId: str = '', instId: str = '', instType: str = '',
                                after: str = '', before: str = '', limit: str = '', proxies={}, proxy_host: str = None):
        '''
        GET /api/v5/tradingBot/grid/orders-algo-history
        https://www.okx.com/docs-v5/zh/#order-book-trading-grid-trading-get-grid-algo-order-history
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        algoOrdType       	String  	是       	策略订单类型grid：现货网格委托contract_grid：合约网格委托moon_grid：天地网格委托
        algoId            	String  	否       	策略订单ID
        instId            	String  	否       	产品ID，如BTC-USDT
        instType          	String  	否       	产品类型SPOT：币币MARGIN：杠杆FUTURES：交割合约SWAP：永续合约
        after             	String  	否       	请求此ID之前（更旧的数据）的分页内容，传的值为对应接口的algoId
        before            	String  	否       	请求此ID之后（更新的数据）的分页内容，传的值为对应接口的algoId
        limit             	String  	否       	返回结果的数量，最大为100，默认100条
        返回参数:
        Parameter         	Type    	Description
        algoId            	String  	策略订单ID
        algoClOrdId       	String  	用户自定义策略ID
        instType          	String  	产品类型
        instId            	String  	产品ID
        cTime             	String  	策略订单创建时间，Unix时间戳的毫秒数格式，如1597026383085
        uTime             	String  	策略订单更新时间，Unix时间戳的毫秒数格式，如1597026383085
        algoOrdType       	String  	策略订单类型grid：现货网格委托contract_grid：合约网格委托moon_grid：天地网格委托
        state             	String  	订单状态stopped：已停止
        rebateTrans       	Array of object	返佣划转信息
        > rebate          	String  	返佣数量
        > rebateCcy       	String  	返佣币种
        triggerParams     	Array of object	信号触发参数
        > triggerAction   	String  	触发行为start：网格启动stop：网格停止
        > triggerStrategy 	String  	触发策略instant：立即触发price：价格触发rsi：rsi指标触发
        > delaySeconds    	String  	延迟触发时间，单位为秒
        > triggerTime     	String  	triggerAction实际触发时间，Unix时间戳的毫秒数格式, 如1597026383085
        > triggerType     	String  	triggerAction的实际触发类型manual：手动触发auto: 自动触发
        > timeframe       	String  	K线种类3m,5m,15m,30m(m代表分钟)1H,4H(H代表小时)1D(D代表天)该字段只在triggerStrategy为rsi时有效
        > thold           	String  	阈值取值[1,100]的整数该字段只在triggerStrategy为rsi时有效
        > triggerCond     	String  	触发条件cross_up：上穿cross_down：下穿above：上方below：下方cross：交叉该字段只在triggerStrategy为rsi时有效
        > timePeriod      	String  	周期14该字段只在triggerStrategy为rsi下有效
        > triggerPx       	String  	触发价格该字段只在triggerStrategy为price下有效
        > stopType        	String  	策略停止类型现货/天地网格1：卖出交易币，2：不卖出交易币合约网格1：停止平仓，2：停止不平仓该字段只在triggerAction为stop时有效
        maxPx             	String  	区间最高价格
        minPx             	String  	区间最低价格
        gridNum           	String  	网格数量
        runType           	String  	网格类型1：等差，2：等比
        tpTriggerPx       	String  	止盈触发价
        slTriggerPx       	String  	止损触发价
        arbitrageNum      	String  	网格套利次数
        totalPnl          	String  	总收益
        pnlRatio          	String  	收益率
        investment        	String  	投入金额现货网格如果投入了交易币则折算为计价币
        gridProfit        	String  	网格利润
        floatProfit       	String  	浮动盈亏
        cancelType        	String  	网格策略停止原因0：无1：手动停止2：止盈停止3：止损停止4：风控停止5：交割停止6: 信号停止
        stopType          	String  	网格策略实际停止类型现货网格/天地网格1：卖出交易币，2：不卖出交易币合约网格1：停止平仓，2：停止不平仓
        quoteSz           	String  	计价币投入数量适用于现货网格/天地网格
        baseSz            	String  	交易币投入数量适用于现货网格
        direction         	String  	合约网格类型long：做多，short：做空，neutral：中性仅适用于合约网格
        basePos           	Boolean 	是否开底仓适用于合约网格
        sz                	String  	投入保证金，单位为USDT适用于合约网格
        lever             	String  	杠杆倍数适用于合约网格
        actualLever       	String  	实际杠杆倍数适用于合约网格
        liqPx             	String  	预估强平价格适用于合约网格
        uly               	String  	标的指数适用于合约网格
        instFamily        	String  	交易品种适用于交割/永续/期权，如BTC-USD适用于合约网格
        ordFrozen         	String  	挂单占用适用于合约网格
        availEq           	String  	可用保证金适用于合约网格
        tag               	String  	订单标签
        profitSharingRatio	String  	分润比例取值范围[0,0.3]如果是普通订单（既不是带单也不是跟单），该字段返回""
        copyType          	String  	分润订单类型0：普通订单1：普通跟单2：分润跟单3：带单
        '''
        return self.send_request(*_GridTradeEndpoints.get_orders_algo_history, **to_local(locals()))

    # GET / 获取网格策略委托订单详情
    def get_orders_algo_details(self, algoOrdType: str, algoId: str, proxies={}, proxy_host: str = None):
        '''
        GET /api/v5/tradingBot/grid/orders-algo-details
        https://www.okx.com/docs-v5/zh/#order-book-trading-grid-trading-get-grid-algo-order-details
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        algoOrdType       	String  	是       	策略订单类型grid：现货网格委托contract_grid：合约网格委托moon_grid：天地网格委托
        algoId            	String  	是       	策略订单ID
        返回参数:
        Parameter         	Type    	Description
        algoId            	String  	策略订单ID
        algoClOrdId       	String  	用户自定义策略ID
        instType          	String  	产品类型
        instId            	String  	产品ID
        cTime             	String  	策略订单创建时间，Unix时间戳的毫秒数格式，如1597026383085
        uTime             	String  	策略订单更新时间，Unix时间戳的毫秒数格式，如1597026383085
        algoOrdType       	String  	策略订单类型grid：现货网格委托contract_grid：合约网格委托moon_grid：天地网格委托
        state             	String  	订单状态starting：启动中running：运行中stopping：终止中no_close_position：已停止未平仓（仅适用于合约网格）stopped：已停止
        rebateTrans       	Array of object	返佣划转信息
        > rebate          	String  	返佣数量
        > rebateCcy       	String  	返佣币种
        triggerParams     	Array of object	信号触发参数
        > triggerAction   	String  	触发行为start：网格启动stop：网格停止
        > triggerStrategy 	String  	触发策略instant：立即触发price：价格触发rsi：rsi指标触发
        > delaySeconds    	String  	延迟触发时间，单位为秒
        > triggerTime     	String  	triggerAction实际触发时间，Unix时间戳的毫秒数格式, 如1597026383085
        > triggerType     	String  	triggerAction的实际触发类型manual：手动触发auto: 自动触发
        > timeframe       	String  	K线种类3m,5m,15m,30m(m代表分钟)1H,4H(H代表小时)1D(D代表天)该字段只在triggerStrategy为rsi时有效
        > thold           	String  	阈值取值[1,100]的整数该字段只在triggerStrategy为rsi时有效
        > triggerCond     	String  	触发条件cross_up：上穿cross_down：下穿above：上方below：下方cross：交叉该字段只在triggerStrategy为rsi时有效
        > timePeriod      	String  	周期14该字段只在triggerStrategy为rsi下有效
        > triggerPx       	String  	触发价格该字段只在triggerStrategy为price下有效
        > stopType        	String  	策略停止类型现货/天地网格1：卖出交易币，2：不卖出交易币合约网格1：停止平仓，2：停止不平仓该字段只在triggerAction为stop时有效
        maxPx             	String  	区间最高价格
        minPx             	String  	区间最低价格
        gridNum           	String  	网格数量
        runType           	String  	网格类型1：等差，2：等比
        tpTriggerPx       	String  	止盈触发价
        slTriggerPx       	String  	止损触发价
        tradeNum          	String  	挂单成交次数
        arbitrageNum      	String  	网格套利次数
        singleAmt         	String  	单网格买卖量
        perMinProfitRate  	String  	预期单网格最低利润率
        perMaxProfitRate  	String  	预期单网格最高利润率
        runPx             	String  	启动时价格
        totalPnl          	String  	总收益
        pnlRatio          	String  	收益率
        investment        	String  	投入金额现货网格如果投入了交易币则折算为计价币
        gridProfit        	String  	网格利润
        floatProfit       	String  	浮动盈亏
        totalAnnualizedRate	String  	总年化
        annualizedRate    	String  	网格年化
        cancelType        	String  	网格策略停止原因0：无1：手动停止2：止盈停止3：止损停止4：风控停止5：交割停止6: 信号停止
        stopType          	String  	网格策略停止类型现货网格/天地网格1：卖出交易币，2：不卖出交易币合约网格1：市价全平，2：停止不平仓
        activeOrdNum      	String  	子订单挂单数量
        quoteSz           	String  	计价币投入数量仅适用于现货网格/天地网格
        baseSz            	String  	交易币投入数量仅适用于现货网格
        curQuoteSz        	String  	当前持有的计价币资产仅适用于现货网格/天地网格
        curBaseSz         	String  	当前持有的交易币资产仅适用于现货网格/天地网格
        profit            	String  	当前可提取利润,单位是计价币仅适用于现货网格/天地网格
        stopResult        	String  	策略停止结果0：默认，1：市价卖币成功-1：市价卖币失败仅适用于现货网格/天地网格
        direction         	String  	合约网格类型long：做多，short：做空，neutral：中性仅适用于合约网格
        basePos           	Boolean 	是否开底仓仅适用于合约网格
        sz                	String  	投入保证金，单位为USDT仅适用于合约网格
        lever             	String  	杠杆倍数仅适用于合约网格
        actualLever       	String  	实际杠杆倍数仅适用于合约网格
        liqPx             	String  	预估强平价格仅适用于合约网格
        uly               	String  	标的指数仅适用于合约网格
        instFamily        	String  	交易品种适用于交割/永续/期权，如BTC-USD适用于合约网格
        ordFrozen         	String  	挂单占用适用于合约网格
        availEq           	String  	可用保证金适用于合约网格
        eq                	String  	策略账户总权益仅适用于合约网格
        tag               	String  	订单标签
        profitSharingRatio	String  	分润比例取值范围[0,0.3]如果是普通订单（既不是带单也不是跟单），该字段返回""
        copyType          	String  	分润订单类型0：普通订单1：普通跟单2：分润跟单3：带单
        '''
        return self.send_request(*_GridTradeEndpoints.get_orders_algo_details, **to_local(locals()))

    # GET / 获取网格策略委托子订单信息
    def get_sub_orders(self, algoId: str, algoOrdType: str, type: str, groupId: str = '', after: str = '',
                       before: str = '', limit: str = '', proxies={}, proxy_host: str = None):
        '''
        GET /api/v5/tradingBot/grid/sub-orders
        https://www.okx.com/docs-v5/zh/#order-book-trading-grid-trading-get-grid-algo-sub-orders
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        algoId            	String  	是       	策略订单ID
        algoOrdType       	String  	是       	策略订单类型grid：现货网格委托contract_grid：合约网格委托moon_grid：天地网格委托
        type              	String  	是       	子订单状态live：未成交filled：已成交
        groupId           	String  	否       	组ID
        after             	String  	否       	请求此ID之前（更旧的数据）的分页内容，传的值为对应接口的ordId
        before            	String  	否       	请求此ID之后（更新的数据）的分页内容，传的值为对应接口的ordId
        limit             	String  	否       	返回结果的数量，最大为100，默认100条
        返回参数:
        Parameter         	Type    	Description
        algoId            	String  	策略订单ID
        algoClOrdId       	String  	用户自定义策略ID
        instType          	String  	产品类型
        instId            	String  	产品ID
        algoOrdType       	String  	策略订单类型grid：现货网格委托contract_grid：合约网格委托moon_grid：天地网格委托
        groupId           	String  	组ID
        ordId             	String  	子订单ID
        cTime             	String  	子订单创建时间，Unix时间戳的毫秒数格式，如1597026383085
        uTime             	String  	子订单更新时间，Unix时间戳的毫秒数格式，如1597026383085
        tdMode            	String  	子订单交易模式cross：全仓isolated：逐仓cash：非保证金
        ccy               	String  	保证金币种仅适用于单币种保证金模式下的全仓杠杆订单
        ordType           	String  	子订单类型market：市价单limit：限价单
        sz                	String  	子订单委托数量
        state             	String  	子订单状态canceled：撤单成功live：等待成交partially_filled：部分成交filled：完全成交cancelling：撤单中
        side              	String  	子订单订单方向buy：买sell：卖
        px                	String  	子订单委托价格
        fee               	String  	子订单手续费数量
        feeCcy            	String  	子订单手续费币种
        rebate            	String  	子订单返佣数量
        rebateCcy         	String  	子订单返佣币种
        avgPx             	String  	子订单平均成交价格
        accFillSz         	String  	子订单累计成交数量
        posSide           	String  	子订单持仓方向long：开平仓模式开多short：开平仓模式开空net：买卖模式
        pnl               	String  	子订单收益
        ctVal             	String  	合约面值仅支持FUTURES/SWAP
        lever             	String  	杠杆倍数
        tag               	String  	订单标签
        '''
        return self.send_request(*_GridTradeEndpoints.get_sub_orders, **to_local(locals()))

    # GET / 获取网格策略委托持仓
    def get_positions(self, algoOrdType: str, algoId: str, proxies={}, proxy_host: str = None):
        '''
        GET /api/v5/tradingBot/grid/positions
        https://www.okx.com/docs-v5/zh/#order-book-trading-grid-trading-get-grid-algo-order-positions
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        algoOrdType       	String  	是       	订单类型contract_grid：合约网格委托
        algoId            	String  	是       	策略订单ID
        返回参数:
        Parameter         	Type    	Description
        algoId            	String  	策略订单ID
        algoClOrdId       	String  	用户自定义策略ID
        instType          	String  	产品类型
        instId            	String  	产品ID，如BTC-USDT-SWAP
        cTime             	String  	策略订单创建时间，Unix时间戳的毫秒数格式，如1597026383085
        uTime             	String  	策略订单更新时间，Unix时间戳的毫秒数格式，如1597026383085
        avgPx             	String  	开仓均价
        ccy               	String  	保证金币种
        lever             	String  	杠杆倍数
        liqPx             	String  	预估强平价
        posSide           	String  	持仓方向net：买卖模式
        pos               	String  	持仓数量
        mgnMode           	String  	保证金模式cross：全仓isolated：逐仓
        mgnRatio          	String  	保证金率
        imr               	String  	初始保证金
        mmr               	String  	维持保证金
        upl               	String  	未实现收益
        uplRatio          	String  	未实现收益率
        last              	String  	最新成交价
        notionalUsd       	String  	仓位美金价值
        adl               	String  	信号区分为5档，从1到5，数字越小代表adl强度越弱
        markPx            	String  	标记价格
        '''
        return self.send_request(*_GridTradeEndpoints.get_positions, **to_local(locals()))

    # POST / 现货/天地网格提取利润
    def set_withdraw_income(self, algoId: str, proxies={}, proxy_host: str = None):
        '''
        POST /api/v5/tradingBot/grid/withdraw-income
        https://www.okx.com/docs-v5/zh/#order-book-trading-grid-trading-post-spot-moon-grid-withdraw-income
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        algoId            	String  	是       	策略订单ID
        返回参数:
        Parameter         	Type    	Description
        algoId            	String  	策略订单ID
        algoClOrdId       	String  	用户自定义策略ID
        profit            	String  	提取的利润
        '''
        return self.send_request(*_GridTradeEndpoints.set_withdraw_income, **to_local(locals()))

    # POST / 调整保证金计算
    def set_compute_margin_balance(self, algoId: str, type: str, amt: str = '', proxies={}, proxy_host: str = None):
        '''
        POST /api/v5/tradingBot/grid/compute-margin-balance
        https://www.okx.com/docs-v5/zh/#order-book-trading-grid-trading-post-compute-margin-balance
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        algoId            	String  	是       	策略订单ID
        type              	String  	是       	调整保证金类型add：增加，reduce：减少
        amt               	String  	否       	调整保证金数量
        返回参数:
        Parameter         	Type    	Description
        maxAmt            	String  	最多可调整的保证金数量
        lever             	String  	调整保证金后的杠杠倍数
        '''
        return self.send_request(*_GridTradeEndpoints.set_compute_margin_balance, **to_local(locals()))

    # POST / 调整保证金
    def set_margin_balance(self, algoId: str, type: str, amt: str = '', percent: str = '', proxies={},
                           proxy_host: str = None):
        '''
        POST /api/v5/tradingBot/grid/margin-balance
        https://www.okx.com/docs-v5/zh/#order-book-trading-grid-trading-post-adjust-margin-balance
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        algoId            	String  	是       	策略订单ID
        type              	String  	是       	调整保证金类型add：增加，reduce：减少
        amt               	String  	可选      	调整保证金数量amt和percent必须传一个
        percent           	String  	可选      	调整保证金百分比
        返回参数:
        Parameter         	Type    	Description
        algoId            	String  	策略订单ID
        algoClOrdId       	String  	用户自定义策略ID
        '''
        return self.send_request(*_GridTradeEndpoints.set_margin_balance, **to_local(locals()))

    # GET / 网格策略智能回测（公共）
    def get_ai_param(self, algoOrdType: str, instId: str, direction: str = '', duration: str = '', proxies={},
                     proxy_host: str = None):
        '''
        公共接口无须鉴权
        https://www.okx.com/docs-v5/zh/#order-book-trading-grid-trading-get-grid-ai-parameter-public
        
        限速：20次/2s
        限速规则：IP
    
        请求参数:
        Parameter         	Type    	Required	Description

        algoOrdType       	String  	是       	策略订单类型grid：现货网格委托contract_grid：合约网格委托moon_grid：天地网格委托
        instId            	String  	是       	产品ID，如BTC-USDT
        direction         	String  	可选      	合约网格类型long：做多，short：做空，neutral：中性合约网格必填
        duration          	String  	否       	回测周期7D：7天，30D：30天，180D：180天默认现货网格为7D，天地网格为180D合约网格只支持7D
        返回参数:
        Parameter         	Type    	Description
        instId            	String  	产品ID
        algoOrdType       	String  	策略订单类型grid：现货网格委托contract_grid：合约网格委托moon_grid：天地网格委托
        duration          	String  	回测周期7D：7天，30D：30天，180D：180天
        gridNum           	String  	网格数量
        maxPx             	String  	区间最高价格
        minPx             	String  	区间最低价格
        perMaxProfitRate  	String  	单网格最高利润率
        perMinProfitRate  	String  	单网格最低利润率
        annualizedRate    	String  	网格年化收益率
        minInvestment     	String  	最小投资数量
        ccy               	String  	投资币种
        runType           	String  	网格类型1：等差，2：等比
        direction         	String  	合约网格类型仅适用于合约网格
        lever             	String  	杠杆倍数仅适用于合约网格
        '''
        return self.send_request(*_GridTradeEndpoints.get_ai_param, **to_local(locals()))

    # POST / 计算最小投资数量（公共）
    def set_min_investment(self, instId: str, algoOrdType: str, gridNum: str, maxPx: str, minPx: str, runType: str,
                           direction: str = '', lever: str = '', basePos: bool = '', investmentData: object = '',
                           proxies={}, proxy_host: str = None):
        '''
        公共接口无须鉴权
        https://www.okx.com/docs-v5/zh/#order-book-trading-grid-trading-post-compute-min-investment-public
        
        限速：20次/2s
        限速规则：IP
    
        请求参数:
        Parameter         	Type    	Required	Description

        instId            	String  	是       	产品ID，如BTC-USDT
        algoOrdType       	String  	是       	策略订单类型grid：现货网格委托contract_grid：合约网格委托
        gridNum           	String  	是       	网格数量
        maxPx             	String  	是       	区间最高价格
        minPx             	String  	是       	区间最低价格
        runType           	String  	是       	网格类型1：等差，2：等比
        direction         	String  	可选      	合约网格类型long：做多，short：做空，neutral：中性适用于合约网格
        lever             	String  	可选      	杠杆倍数适用于合约网格
        basePos           	Boolean 	否       	是否开底仓默认为false
        investmentData    	Array of object	否       	投资信息
        > amt             	String  	是       	投资数量
        > ccy             	String  	是       	投资币种
        返回参数:
        Parameter         	Type    	Description
        investmentData    	Array of object	最小投入信息
        > amt             	String  	最小投入数量
        > ccy             	String  	最小投入币种
        singleAmt         	String  	单网格买卖量现货网格单位为计价币合约网格单位为张
        '''
        return self.send_request(*_GridTradeEndpoints.set_min_investment, **to_local(locals()))

    # GET / RSI回测（公共）
    def get_rsi_back_testing(self, instId: str, timeframe: str, thold: str, timePeriod: str, triggerCond: str = '',
                             duration: str = '', proxies={}, proxy_host: str = None):
        '''
        公共接口无须鉴权
        https://www.okx.com/docs-v5/zh/#order-book-trading-grid-trading-get-rsi-back-testing-public
        
        限速：20次/2s
        限速规则：IP
    
        请求参数:
        Parameter         	Type    	Required	Description

        instId            	String  	是       	产品ID，如BTC-USDT适用于币币
        timeframe         	String  	是       	K线种类3m,5m,15m,30m(m代表分钟)1H,4H(H代表小时)1D(D代表天)
        thold             	String  	是       	阈值取值[1,100]的整数
        timePeriod        	String  	是       	周期14
        triggerCond       	String  	否       	触发条件cross_up：上穿cross_down：下穿above：上方below：下方cross：交叉默认是cross_down
        duration          	String  	否       	回测周期1M：1个月默认1M
        返回参数:
        Parameter         	Type    	Description
        triggerNum        	String  	触发次数
        '''
        return self.send_request(*_GridTradeEndpoints.get_rsi_back_testing, **to_local(locals()))



from okx.api.market import Market
from okx.api.public import Public


class MarketBase():
    def __init__(
            self,
            instType: str,
            key: str = '',
            secret: str = '',
            passphrase: str = '',
            flag: str = '0',
            timezone='Asia/Shanghai',
            proxies={},
            proxy_host: str = None,
    ):
        self.instType = instType.upper()

        self.marketAPI = Market(
            key=key,
            secret=secret,
            passphrase=passphrase,
            flag=flag,
            proxies=proxies,
            proxy_host=proxy_host,
        )
        self.publicAPI = Public(
            key=key,
            secret=secret,
            passphrase=passphrase,
            flag=flag,
            proxies=proxies,
            proxy_host=proxy_host,
        )
        self.timezone = timezone



'''
子账户
https://www.okx.com/docs-v5/zh/#sub-account-rest-api
'''
from paux.param import to_local
from okx.api._client import Client


class _SubAccountEndpoints():
    get_list = ['/api/v5/users/subaccount/list', 'GET']  # 查看子账户列表
    set_modify_apikey = ['/api/v5/users/subaccount/modify-apikey', 'POST']  # 重置子账户的APIKey
    get_account_balances = ['/api/v5/account/subaccount/balances', 'GET']  # 获取子账户交易账户余额
    get_assert_balances = ['/api/v5/asset/subaccount/balances', 'GET']  # 获取子账户资金账户余额
    get_max_withdrawal = ['/api/v5/account/subaccount/max-withdrawal', 'GET']  # 获取子账户最大可转余额
    get_bills = ['/api/v5/asset/subaccount/bills', 'GET']  # 查询子账户转账记录
    get_managed_subaccount_bills = ['/api/v5/asset/subaccount/managed-subaccount-bills', 'GET']  # 查询托管子账户转账记录
    set_transfer = ['/api/v5/asset/subaccount/transfer', 'POST']  # 子账户间资金划转
    set_transfer_out = ['/api/v5/users/subaccount/set-transfer-out', 'POST']  # 设置子账户主动转出权限
    get_entrust_subaccount_list = ['/api/v5/users/entrust-subaccount-list', 'GET']  # 查看被托管的子账户列表
    get_if_rebate = ['/api/v5/users/partner/if-rebate', 'GET']  # 获取用户的节点返佣信息
    set_loan_allocation = ['/api/v5/account/subaccount/set-loan-allocation', 'POST']  # 设置子账户尊享借币比率
    get_interest_limits = ['/api/v5/account/subaccount/interest-limits', 'GET']  # 获取子账户借币利率与限额


class SubAccount(Client):

    # 查看子账户列表
    def get_list(self, enable: str = '', subAcct: str = '', after: str = '', before: str = '', limit: str = '',
                 proxies={}, proxy_host: str = None):
        '''
        仅适用于母账户
        https://www.okx.com/docs-v5/zh/#sub-account-rest-api-get-sub-account-list
        
        限速：2次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        enable            	String  	否       	子账户状态true: 正常使用false: 冻结
        subAcct           	String  	否       	子账户名称
        after             	String  	否       	查询在此之前的内容，值为时间戳，Unix时间戳为毫秒数格式
        before            	String  	否       	查询在此之后的内容，值为时间戳，Unix时间戳为毫秒数格式
        limit             	String  	否       	分页返回的结果集数量，最大为100，不填默认返回100条
        返回参数:
        Parameter         	Type    	Description
        type              	String  	子账户类型1: 普通子账户2: 资管子账户5: 托管子账户 - Copper
        enable            	Boolean 	子账户状态true: 正常使用false: 冻结
        subAcct           	String  	子账户名称
        uid               	String  	子账户UID
        label             	String  	子账户备注
        mobile            	String  	子账户绑定手机号
        gAuth             	Boolean 	子账户是否开启的登录时的谷歌验证true: 已开启false: 未开启
        canTransOut       	Boolean 	是否可以主动转出true: 可以转出false: 不可转出
        ts                	String  	子账户创建时间，Unix时间戳的毫秒数格式 ，如1597026383085
        '''
        return self.send_request(*_SubAccountEndpoints.get_list, **to_local(locals()))

    # 重置子账户的APIKey
    def set_modify_apikey(self, subAcct: str, apiKey: str, label: str = '', perm: str = '', ip: str = '', proxies={},
                          proxy_host: str = None):
        '''
        仅适用于母账户,且母账户APIKey必须绑定IP
        https://www.okx.com/docs-v5/zh/#sub-account-rest-api-reset-the-api-key-of-a-sub-account
        
        限速：1次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        subAcct           	String  	是       	子账户名称
        apiKey            	String  	是       	子账户API的公钥
        label             	String  	否       	子账户APIKey的备注，如果填写该字段，则该字段会被重置
        perm              	String  	否       	子账户APIKey权限read_only：读取trade：交易多个权限用半角逗号隔开。如果填写该字段，则该字段会被重置。
        ip                	String  	否       	子账户APIKey绑定ip地址，多个ip用半角逗号隔开，最多支持20个ip。如果填写该字段，那该字段会被重置。如果ip传""，则表示解除IP绑定。
        返回参数:
        Parameter         	Type    	Description
        subAcct           	String  	子账户名称
        label             	String  	APIKey的备注
        apiKey            	String  	API公钥
        perm              	String  	APIKey权限
        ip                	String  	APIKey绑定的ip地址
        ts                	String  	创建时间
        '''
        return self.send_request(*_SubAccountEndpoints.set_modify_apikey, **to_local(locals()))

    # 获取子账户交易账户余额
    def get_account_balances(self, subAcct: str, proxies={}, proxy_host: str = None):
        '''
        获取子账户交易账户余额（适用于母账户）
        https://www.okx.com/docs-v5/zh/#sub-account-rest-api-get-sub-account-trading-balance
        
        限速：6次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        subAcct           	String  	是       	子账户名称
        返回参数:
        Parameter         	Type    	Description
        uTime             	String  	获取账户信息的最新时间，Unix时间戳的毫秒数格式，如1597026383085
        totalEq           	String  	美金层面权益
        isoEq             	String  	美金层面逐仓仓位权益适用于单币种保证金模式和跨币种保证金模式和组合保证金模式
        adjEq             	String  	美金层面有效保证金适用于跨币种保证金模式和组合保证金模式
        ordFroz           	String  	美金层面全仓挂单占用保证金适用于跨币种保证金模式和组合保证金模式
        imr               	String  	美金层面占用保证金适用于跨币种保证金模式和组合保证金模式
        mmr               	String  	美金层面维持保证金适用于跨币种保证金模式和组合保证金模式
        borrowFroz        	String  	账户美金层面潜在借币占用保证金仅适用于跨币种保证金模式和组合保证金模式. 在其他账户模式下为"".
        mgnRatio          	String  	美金层面保证金率适用于跨币种保证金模式和组合保证金模式
        notionalUsd       	String  	以美金价值为单位的持仓数量，即仓位美金价值适用于跨币种保证金模式和组合保证金模式
        details           	Array   	各币种资产详细信息
        > ccy             	String  	币种
        > eq              	String  	币种总权益
        > cashBal         	String  	币种余额
        > uTime           	String  	币种余额信息的更新时间，Unix时间戳的毫秒数格式，如1597026383085
        > isoEq           	String  	币种逐仓仓位权益适用于单币种保证金模式和跨币种保证金模式
        > availEq         	String  	可用保证金适用于单币种保证金模式和跨币种保证金模式
        > disEq           	String  	美金层面币种折算权益
        > availBal        	String  	可用余额适用于简单交易模式
        > frozenBal       	String  	币种占用金额
        > ordFrozen       	String  	挂单冻结数量
        > liab            	String  	币种负债额适用于跨币种保证金模式和组合保证金模式
        > upl             	String  	未实现盈亏适用于单币种保证金模式和跨币种保证金模式和组合保证金模式
        > uplLiab         	String  	由于仓位未实现亏损导致的负债适用于跨币种保证金模式和组合保证金模式
        > crossLiab       	String  	币种全仓负债额适用于跨币种保证金模式和组合保证金模式
        > isoLiab         	String  	币种逐仓负债额适用于跨币种保证金模式和组合保证金模式
        > mgnRatio        	String  	保证金率适用于单币种保证金模式
        > interest        	String  	计息适用于跨币种保证金模式和组合保证金模式
        > twap            	String  	当前负债币种触发系统自动换币的风险0、1、2、3、4、5其中之一，数字越大代表您的负债币种触发自动换币概率越高适用于跨币种保证金模式和组合保证金模式
        > maxLoan         	String  	币种最大可借适用于跨币种保证金模式和组合保证金模式的全仓
        > eqUsd           	String  	币种权益美金价值
        > borrowFroz      	String  	币种美金层面潜在借币占用保证金仅适用于跨币种保证金模式和组合保证金模式. 在其他账户模式下为"".
        > notionalLever   	String  	币种杠杆倍数适用于单币种保证金模式
        '''
        return self.send_request(*_SubAccountEndpoints.get_account_balances, **to_local(locals()))

    # 获取子账户资金账户余额
    def get_assert_balances(self, subAcct: str, ccy: str = '', proxies={}, proxy_host: str = None):
        '''
        获取子账户资金账户余额（适用于母账户）
        https://www.okx.com/docs-v5/zh/#sub-account-rest-api-get-sub-account-funding-balance
        
        限速：6次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        subAcct           	String  	是       	子账户名称
        ccy               	String  	否       	币种，如BTC支持多币种查询（不超过20个），币种之间半角逗号分隔
        返回参数:
        Parameter         	Type    	Description
        ccy               	String  	币种，如BTC
        bal               	String  	余额
        frozenBal         	String  	冻结（不可用）
        availBal          	String  	可用余额
        '''
        return self.send_request(*_SubAccountEndpoints.get_assert_balances, **to_local(locals()))

    # 获取子账户最大可转余额
    def get_max_withdrawal(self, subAcct: str, ccy: str = '', proxies={}, proxy_host: str = None):
        '''
        获取子账户最大可转余额（适用于母账户）。不指定币种会返回所有拥有的币种资产可划转数量。
        https://www.okx.com/docs-v5/zh/#sub-account-rest-api-get-sub-account-maximum-withdrawals
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        subAcct           	String  	是       	子账户名称
        ccy               	String  	否       	币种，如BTC支持多币种查询（不超过20个），币种之间半角逗号分隔
        返回参数:
        Parameter         	Type    	Description
        ccy               	String  	币种
        maxWd             	String  	最大可划转数量（不包含跨币种保证金模式借币金额）
        maxWdEx           	String  	最大可划转数量（包含跨币种保证金模式借币金额）
        spotOffsetMaxWd   	String  	现货对冲不支持借币最大可转数量仅适用于组合保证金模式
        spotOffsetMaxWdEx 	String  	现货对冲支持借币的最大可转数量仅适用于组合保证金模式
        '''
        return self.send_request(*_SubAccountEndpoints.get_max_withdrawal, **to_local(locals()))

    # 查询子账户转账记录
    def get_bills(self, ccy: str = '', type: str = '', subAcct: str = '', after: str = '', before: str = '',
                  limit: str = '', proxies={}, proxy_host: str = None):
        '''
        仅适用于母账户
        https://www.okx.com/docs-v5/zh/#sub-account-rest-api-get-history-of-sub-account-transfer
        
        限速：6次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	否       	币种，如 BTC
        type              	String  	否       	划转类型0：母账户转子账户1：子账户转母账户
        subAcct           	String  	否       	子账户名称
        after             	String  	否       	查询在此之前的内容，值为时间戳，Unix时间戳的毫秒数格式，如1597026383085
        before            	String  	否       	查询在此之后的内容，值为时间戳，Unix时间戳的毫秒数格式，如1597026383085
        limit             	String  	否       	分页返回的结果集数量，最大为100，不填默认返回100条
        返回参数:
        Parameter         	Type    	Description
        billId            	String  	账单ID
        ccy               	String  	账户余额币种
        amt               	String  	划转金额
        type              	String  	账单类型
        subAcct           	String  	子账户名称
        ts                	String  	账单ID创建时间，Unix时间戳的毫秒数格式，如1597026383085
        '''
        return self.send_request(*_SubAccountEndpoints.get_bills, **to_local(locals()))

    # 查询托管子账户转账记录
    def get_managed_subaccount_bills(self, ccy: str = '', type: str = '', subAcct: str = '', subUid: str = '',
                                     after: str = '', before: str = '', limit: str = '', proxies={},
                                     proxy_host: str = None):
        '''
        仅适用于交易团队母账户查看托管给自己的托管子账户转账记录
        https://www.okx.com/docs-v5/zh/#sub-account-rest-api-get-history-of-managed-sub-account-transfer
        
        限速：6次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	否       	币种，如BTC
        type              	String  	否       	划转类型0：母账户转子账户1：子账户转母账户
        subAcct           	String  	否       	子账户名称
        subUid            	String  	否       	子账户UID
        after             	String  	否       	查询在此之前的内容，值为时间戳，Unix时间戳为毫秒数格式，如1597026383085
        before            	String  	否       	查询在此之后的内容，值为时间戳，Unix时间戳为毫秒数格式，如1597026383085
        limit             	String  	否       	分页返回的结果集数量，最大为100，默认返回100条
        返回参数:
        Parameter         	Type    	Description
        billId            	String  	账单ID
        ccy               	String  	账户余额币种
        amt               	String  	划转金额
        type              	String  	账单类型
        subAcct           	String  	子账户名称
        subUid            	String  	子账户UID
        ts                	String  	账单ID创建时间，Unix时间戳的毫秒数格式，如1597026383085
        '''
        return self.send_request(*_SubAccountEndpoints.get_managed_subaccount_bills, **to_local(locals()))

    # 子账户间资金划转
    def set_transfer(self, ccy: str, amt: str, from_: str, to: str, fromSubAccount: str, toSubAccount: str,
                     loanTrans: bool = '', omitPosRisk: str = '', proxies={}, proxy_host: str = None):
        '''
        母账户控制子账户与子账户之间划转（仅适用于母账户）
        https://www.okx.com/docs-v5/zh/#sub-account-rest-api-master-accounts-manage-the-transfers-between-sub-accounts
        
        限速：1次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        ccy               	String  	是       	币种
        amt               	String  	是       	划转数量
        from              	String  	是       	转出子账户类型6：资金账户18：交易账户
        to                	String  	是       	转出子账户类型6：资金账户18：交易账户
        fromSubAccount    	String  	是       	转出子账户的子账户名称
        toSubAccount      	String  	是       	转入子账户的子账户名称
        loanTrans         	Boolean 	否       	是否支持跨币种保证金模式或组合保证金模式下的借币转入/转出true 或 false，默认false
        omitPosRisk       	String  	否       	是否忽略仓位风险默认为false仅适用于组合保证金模式
        返回参数:
        Parameter         	Type    	Description
        transId           	String  	划转ID
        '''
        data = to_local(locals())
        data['from'] = data['from_']
        del data['from_']
        return self.send_request(*_SubAccountEndpoints.set_transfer, **data)

    # 设置子账户主动转出权限
    def set_transfer_out(self, subAcct: str, canTransOut: bool = '', proxies={}, proxy_host: str = None):
        '''
        设置子账户转出权限（仅适用于母账户），默认可转出至母账户。
        https://www.okx.com/docs-v5/zh/#sub-account-rest-api-set-permission-of-transfer-out
        
        限速：1次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        subAcct           	String  	是       	子账户名称，支持设置多个（不超过20个），子账户名称之间半角逗号分隔
        canTransOut       	Boolean 	否       	是否可以主动转出，默认为truefalse：不可转出true：可以转出
        返回参数:
        Parameter         	Type    	Description
        subAcct           	String  	子账户名称
        canTransOut       	Boolean 	是否可以主动转出false：不可转出true：可以转出
        '''
        return self.send_request(*_SubAccountEndpoints.set_transfer_out, **to_local(locals()))

    # 查看被托管的子账户列表
    def get_entrust_subaccount_list(self, subAcct: str = '', proxies={}, proxy_host: str = None):
        '''
        交易团队使用该接口查看当前托管中的子账户列表
        https://www.okx.com/docs-v5/zh/#sub-account-rest-api-get-custody-trading-sub-account-list
        
        限速：1次/s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        subAcct           	String  	否       	子账户名称
        返回参数:
        Parameter         	Type    	Description
        subAcct           	String  	子账户名称
        '''
        return self.send_request(*_SubAccountEndpoints.get_entrust_subaccount_list, **to_local(locals()))

    # 获取用户的节点返佣信息
    def get_if_rebate(self, apiKey: str, proxies={}, proxy_host: str = None):
        '''
        该接口用于节点查询用户的返佣信息
        https://www.okx.com/docs-v5/zh/#sub-account-rest-api-get-the-user-39-s-affiliate-rebate-information
        
        限速：20次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        apiKey            	String  	是       	用户的 API key，仅支持使用被邀请人母账号的 API key
        返回参数:
        Parameter         	Type    	Description
        result            	Boolean 	用户是否与当前节点有邀请关系。true,false
        type              	String  	是否有节点返佣0有节点返佣1没有节点返佣，因为调用该接口的账户不是节点身份2没有节点返佣，因为不存在邀请/召回关系，如：api key不存在4没有节点返佣，因为用户的手续费等级大于等于VIP6
        '''
        return self.send_request(*_SubAccountEndpoints.get_if_rebate, **to_local(locals()))

    # 设置子账户尊享借币比率
    def set_loan_allocation(self, enable: bool, alloc: object = '', proxies={}, proxy_host: str = None):
        '''
        为子账户设置 VIP 尊享借币可用额度的比率（百分比），仅适用于有交易权限的母账户 API key.
        https://www.okx.com/docs-v5/zh/#sub-account-rest-api-set-sub-accounts-vip-loan-allocation
        
        限速：5次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        enable            	Boolean 	是       	为子账户设置 VIP 尊享借币可用额度的比率。true或者false
        alloc             	Array of objects	否       	如果 enable = false, 将不会验证该参数
        > subAcct         	String  	是       	子账户名称
        > loanAlloc       	String  	是       	子账户尊享借币可用额度的比率（百分比）范围为[0, 100]. 精度为 0.01% (2位小数)"0" 代表取消子账户的 VIP 借币额度。
        返回参数:
        Parameter         	Type    	Description
        result            	String  	请求结果，枚举值为true,false
        '''
        return self.send_request(*_SubAccountEndpoints.set_loan_allocation, **to_local(locals()))

    # 获取子账户借币利率与限额
    def get_interest_limits(self, subAcct: str, ccy: str = '', proxies={}, proxy_host: str = None):
        '''
        仅适用于母账户的 API key, 仅返回尊享借币信息。
        https://www.okx.com/docs-v5/zh/#sub-account-rest-api-get-sub-account-borrow-interest-and-limit
        
        限速：5次/2s
        限速规则：UserID
    
        请求参数:
        Parameter         	Type    	Required	Description

        subAcct           	String  	是       	子账户名称， 只能填写一个
        ccy               	String  	否       	借贷币种，如BTC
        返回参数:
        Parameter         	Type    	Description
        subAcct           	String  	子账户名称
        interest          	String  	可忽略，只返回 “”
        nextDiscountTime  	String  	下次扣息时间，Unix时间戳的毫秒数格式，如1597026383085
        nextInterestTime  	String  	下次计息时间，Unix时间戳的毫秒数格式，如1597026383085
        loanAlloc         	String  	当前交易账户尊享借币可用额度的比率（百分比）1. 范围为[0, 100]. 精度为 0.01% (2位小数)2. 0 代表母账户没有为子账户分配；3. "" 代表母子账户共享
        records           	Array   	各币种详细信息
        > ccy             	String  	借贷币种，如BTC
        > rate            	String  	日利率
        > loanQuota       	String  	母账户维度借币限额如果已配置可用额度，该字段代表当前交易账户（子账户）的借币限额
        > surplusLmt      	String  	母子账户剩余可借如果已配置可用额度，该字段代表当前交易账户（子账户）的剩余可借
        > surplusLmtDetails	Array   	母子账户剩余可借额度详情，母子账户剩余可借额度的值取该数组中的最小值，可以用来判断是什么原因导致可借额度不足仅适用于尊享借币
        >> allAcctRemainingQuota	String  	母子账户剩余额度
        >> curAcctRemainingQuota	String  	当前子账户剩余额度仅适用于为子账户分配限额的场景
        >> platRemainingQuota	String  	平台剩余额度，当平台剩余额度大于curAcctRemainingQuota或者allAcctRemainingQuota时，会显示大于某个值，如">1000"
        > usedLmt         	String  	母子账户已借额度如果已配置可用额度，该字段代表当前交易账户（子账户）的已借额度
        > interest        	String  	可忽略，只返回 “”
        > posLoan         	String  	当前账户负债占用（子账户，锁定额度内）仅适用于尊享借币
        > availLoan       	String  	当前账户剩余可用（子账户，锁定额度内）仅适用于尊享借币
        > usedLoan        	String  	当前账户（子账户）已借额度仅适用于尊享借币
        > avgRate         	String  	币种已借平均(小时)利率，仅适用于尊享借币
        '''
        return self.send_request(*_SubAccountEndpoints.get_interest_limits, **to_local(locals()))



from typing import Union

from candlelite.crypto.okx_lite import (
    OKX_TIMEZONE,  # 历史K线默认时区
    OKX_CANDLE_DATE_BASE_DIR,  # 历史K线以日期为单位的默认存储路径
    OKX_DEFAULT_BAR  # 历史K线默认时间粒度
)


class CandleRule():
    # 收集的产品 all表示全部正在交易的产品
    INSTIDS: Union[str, list] = 'all'
    # 过滤的产品
    INSTIDS_FILTER: list = []
    # 产品名称中需要包含的内容
    INSTID_CONTAINS = ''
    # 产品名称必须以何内容结尾
    INSTID_ENDSWITH = ''
    # candle_map 更新时间间隔（秒）
    UPDATE_INTERVAL_SECONDS: int = 3
    # candle_map 缓存K线数据长度
    LENGTH: int = 1440 * 2
    # 时间颗粒度
    BAR: str = OKX_DEFAULT_BAR
    # candle_map 保存缓存文件时间间隔（秒） None：不保存缓存
    CACHE_DELAY_SECONDS: Union[int, None] = 60 * 60
    # 服务启动 需维护本地多少天的历史K线数据
    LOCAL_CANDLE_DAYS: int = 2
    # 每日下载昨日历史K线数据的时刻 格式：%H:%M:%S None表示不下载
    DOWNLOAD_TIME: Union[str, None] = '00:10:00'
    # 以天为单位的数据存储路径
    CANDLE_DIR = OKX_CANDLE_DATE_BASE_DIR
    # 缓存数据路径
    CACHE_DIR = './OKX_CACHE'
    # 时区
    TIMEZONE = OKX_TIMEZONE
    # KEY 通常无需填写
    KEY: str = ''
    # SECRET 通常无需填写
    SECRET: str = ''
    # PASSPHRASE 通常无需填写
    PASSPHRASE: str = ''
    # 日志文件夹
    LOG_DIRPATH = './OKX_CANDLE_SERVER_LOG_DATA'
    # 日志文件级别
    LOG_FILE_LEVEL = 'INFO'
    # 日志打印级别
    LOG_CONSOLE_LEVEL = 'DEBUG'



from okx.app.exception._base import AbstractEXP


# 请求方法错误
class RequestMethodError(AbstractEXP):
    pass


# 响应状态码错误
class ResponseStatusError(AbstractEXP):
    pass


# 响应Json数据中的code状态码异常
class CodeException(AbstractEXP):
    pass


# 请求异常
class RequestException(AbstractEXP):
    pass



from okx.app.market._base import MarketBase


class Ticker(MarketBase):

    # 获取所有产品的行情列表
    def get_tickers(self, instIds=[]) -> dict:
        api_tickers_result = self.marketAPI.get_tickers(instType=self.instType)
        if not instIds:
            return api_tickers_result
        if not api_tickers_result['code'] == '0':
            return api_tickers_result
        datas = []
        for _data in api_tickers_result['data']:
            if _data['instId'] in instIds:
                datas.append(_data)
        api_tickers_result['data'] = datas
        return api_tickers_result

    # 获取所有产品的行情字典
    def get_tickersMap(self, instIds=[]) -> dict:
        bookTickers_result = self.get_tickers(instIds=instIds)
        # [ERROR RETURN] 数据异常
        if bookTickers_result['code'] != '0':
            return bookTickers_result
        data_map = {}
        for ticker in bookTickers_result['data']:
            instId = ticker['instId']
            data_map[instId] = ticker

        bookTickers_result['data'] = data_map
        return bookTickers_result

    # 获取单个产品的行情
    def get_ticker(self, instId: str) -> dict:
        result = self.marketAPI.get_ticker(instId=instId)
        if result['code'] != '0':
            return result
        result['data'] = result['data'][0]
        return result

    # 产品深度
    def get_books(self, instId: str, sz: int = 1):
        result = self.marketAPI.get_books(instId=instId, sz=sz)
        if result['code'] != '0':
            return result
        result['data'] = result['data'][0]
        return result

    # 获取产品轻量深度
    def get_books_lite(self, instId: str):
        result = self.marketAPI.get_books_lite(instId=instId)
        if result['code'] != '0':
            return result
        result['data'] = result['data'][0]
        return result
